﻿#define ListOf
// #define IEnumerable // Listing 15-18
// #define IQueryable

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevLeap.Linq.Architecture.NTier.DomainModel;
using DevLeap.Linq.Architecture.NTier.DataLayer;

namespace DevLeap.Linq.Architecture.NTier.BusinessLayer
{
    public class CustomersBiz : BaseBiz
    {
#if ListOf
        private IDal<Customer, List<Customer>, String> _dal;
#endif
#if IEnumerable
        private IDal<Customer, IEnumerable<Customer>, String> _dal;
#endif
#if IQueryable
        private IDal<Customer, IQueryable<Customer>, String> _dal;
#endif

        public CustomersBiz()
        {
#if ListOf
            this._dal = DalFactory.Instance.CreateDal<Customer, List<Customer>, String>();
#endif
#if IEnumerable
            this._dal = DalFactory.Instance.CreateDal<Customer, IEnumerable<Customer>, String>();
#endif
#if IQueryable
            this._dal = DalFactory.Instance.CreateDal<Customer, IQueryable<Customer>, String>();
#endif
        }

        public Customer InsertNewCustomer(Customer item)
        {
            // Check authorization policies

            // Validate the entity content and status

            // Add it to the data persistence storage
            Customer result = this._dal.Add(item);

            // Return the resulting customer instance to the caller
            return (result);
        }

        public Customer UpdateCustomer(Customer item)
        {
            // Check authorization policies

            // Validate the entity content and status

            // Update it into the data persistence storage
            Customer result = this._dal.Update(item);

            // Return the resulting customer instance to the caller
            return (result);
        }

        public Boolean DeleteCustomer(Customer item)
        {
            // Check authorization policies

            // Validate the entity content and status

            // Update it into the data persistence storage
            Boolean result = this._dal.Delete(item);

            // Return the resulting customer instance to the caller
            return (result);
        }

        public Customer GetCustomerByKey(String key)
        {
            // Check authorization policies

            // Add it to the data persistence storage
            Customer result = this._dal.Read(key);

            // Eventually check and personalize the entity for the caller

            // Return the resulting customer instance to the caller
            return (result);
        }

#if ListOf
        public List<Customer> GetCustomersByCountry(String country)
#endif
#if IEnumerable
        public IEnumerable<Customer> GetCustomersByCountry(String country)
#endif
#if IQueryable
        public IQueryable<Customer> GetCustomersByCountry(String country)
#endif
        {
            // Check authorization policies

            // Add it to the data persistence storage
            var result =
                from c in this._dal.ReadAll()
                where c.Country == country
                select c;

            // Eventually check and personalize the entity for the caller

            // Return the resulting customer instance to the caller
#if ListOf
            return (result.ToList());
#endif
#if IEnumerable || IQueryable
            return (result);
#endif
        }

#if ListOf
        public List<Customer> GetAllCustomers()
#endif
#if IEnumerable
        public IEnumerable<Customer> GetAllCustomers()
#endif
#if IQueryable
        public IQueryable<Customer> GetAllCustomers()
#endif
        {
            // Check authorization policies

            // Add it to the data persistence storage
            var result =
                from c in this._dal.ReadAll()
                select c;

            // Eventually check and personalize the entity for the caller

            // Return the resulting customer instance to the caller
#if ListOf
            return (result.ToList());
#endif
#if IEnumerable || IQueryable
            return (result);
#endif
        }


        // Listing 15-15

        public void RefundLegacy() {

            // Temporary repository for delayed orders, grouped by country
            SortedDictionary<String, List<Order>> ordersToRefundByCountry =
                new SortedDictionary<String, List<Order>>();

            // Get the full list of customers
            // In real life be careful on this kind of "full scan" methods!
            var customers = this._dal.ReadAll();
            foreach (var c in customers) {
                foreach (var o in c.Orders) {
                    // If the order is late
                    if (o.RequiredDate < o.ShippedDate) {
                        // Check if the current customer country already exists
                        if (!ordersToRefundByCountry.ContainsKey(c.Country)) {
                            // Otherwise create it
                            ordersToRefundByCountry[c.Country] = new List<Order>();
                        }
                    // Add the order to the list
                    ordersToRefundByCountry[c.Country].Add(o);
                    }
                }
            }

            Int32 itemsCount = 0;
            foreach (var itemsGroup in ordersToRefundByCountry) {
                foreach (var item in itemsGroup.Value) {
                    // Imagine that here you create and send the refund form
                    Console.WriteLine("Country: {0} - CustomerID: {1} - OrderID: {2} - DelayedDays: {3}",
                    itemsGroup.Key, item.CustomerID, item.OrderID,((TimeSpan)item.ShippedDate.Value.Subtract(item.RequiredDate.Value)).Days);
                    itemsCount++;
                }
            }
            

            // Hypothetical output, even if we are in a BIZ component, 
            // thus the output should be elsewhere, but this is just a sample
            Console.WriteLine("Total count of orders to refund: {0}", itemsCount);
        }

        // Listing 15-16

        public void Refund()
        {
            var customersWithOrdersToRefund =
                (from c in this._dal.ReadAll()
                 group c by c.Country into customersByCountry
                     from i in customersByCountry
                     from o in i.Orders
                     where o.RequiredDate < o.ShippedDate
                     orderby i.Country
                         select new
                         {
                             i.Country,
                             i.CustomerID,
                             o.OrderID,
                             DelayedDays =
                             ((TimeSpan)o.ShippedDate.Value.Subtract(o.RequiredDate.Value)).Days
                         }).ToList();

            // Hypothetical output, even if we are in a BIZ component, 
            // thus the output should be elsewhere, but this is just a sample

            foreach (var item in customersWithOrdersToRefund)
            {
                // Imagine that here you create and send the refund form
                Console.WriteLine(item);
            }
            
            Console.WriteLine("Total count of orders to refund: {0}",
                customersWithOrdersToRefund.Count());
        }
    }
}
