﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevLeap.Linq.Architecture.NTier.BusinessLayer;
using DevLeap.Linq.Architecture.NTier.DomainModel;
using System.Runtime.Serialization;
using System.IO;
using System.Xml.Linq;

namespace DevLeap.Linq.Architecture.NTier.ConsoleUI
{
    class Program
    {
        static void Main(string[] args)
        {
            String customerID = "ALFKI";
            Console.WriteLine("Reading customer {0}", customerID);
            Customer alfki = ReadCustomer(customerID);
            dumpCustomer(alfki);
            Console.WriteLine("Press ENTER to move to the next sample...");
            Console.ReadLine();

            Console.WriteLine("Inserting a new customer");
            Customer customer = InsertNewCustomer();
            dumpCustomer(customer);
            Console.WriteLine("Press ENTER to move to the next sample...");
            Console.ReadLine();

            Console.WriteLine("Updating customer {0}", customer.CustomerID);
            customer = UpdateExistingCustomer(customer);
            dumpCustomer(customer);
            Console.WriteLine("Press ENTER to move to the next sample...");
            Console.ReadLine();

            Console.WriteLine("Reading Italian customers");
            ReadItalianCustomers();
            Console.WriteLine("Press ENTER to move to the next sample...");
            Console.ReadLine();

            Console.WriteLine("Deleting customer {0}", customer.CustomerID);
            Boolean deletionResult = DeleteExistingCustomer(customer);
            Console.WriteLine("Deletion result: {0}", deletionResult);
            Console.WriteLine("Press ENTER to exit from the application...");
            Console.ReadLine();
        }

        private static Boolean DeleteExistingCustomer(Customer customer)
        {
            CustomersBiz cb = new CustomersBiz();

            // Delete the new customer
            Boolean deletionResult = cb.DeleteCustomer(customer);

            return (deletionResult);
        }

        // Listing 15-19

        private static void ReadItalianCustomers()
        {
            CustomersBiz cb = new CustomersBiz();

            // Query the full list of customers to extract italian ones
            var query =
                from c in cb.GetCustomersByCountry("Italy")
                select new { c.CustomerID, c.ContactName, c.Country };

            foreach (var c in query)
            {
                Console.WriteLine(c);
            }
        }

        private static Customer UpdateExistingCustomer(Customer customer)
        {
            CustomersBiz cb = new CustomersBiz();

            // Update that customer
            customer.Phone = "+39-030-123456";
            customer.Orders.Add(new Order
            {
                CustomerID = customer.CustomerID,
                ShipName = "Roberto Brunetti"
            });
            customer = cb.UpdateCustomer(customer);

            return (customer);
        }

        private static Customer InsertNewCustomer()
        {
            CustomersBiz cb = new CustomersBiz();

            // Create a new customer
            Customer newCustomer = new Customer();
            newCustomer.CustomerID = "DEVLP";
            newCustomer.CompanyName = "DevLeap";
            newCustomer.ContactName = "Paolo Pialorsi";
            newCustomer.Country = "Italy";
            newCustomer.Orders = new List<Order>(new Order[] { 
                new Order { CustomerID = newCustomer.CustomerID ,
                    ShipName = "Paolo Pialorsi"},
                new Order { CustomerID = newCustomer.CustomerID ,
                    ShipName = "Marco Russo"},
            });

            try
            {
                newCustomer = cb.InsertNewCustomer(newCustomer);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            return (newCustomer);
        }

        private static Customer ReadCustomer(String customerID)
        {
            CustomersBiz cb = new CustomersBiz();

            // Read an existing customer
            Customer customer = cb.GetCustomerByKey(customerID);

            return (customer);
        }

        private static void createNorthwindCustomersWithOrdersXmlFile()
        {
            CustomersBiz cb = new CustomersBiz();

            // Create the NorthwindCustomersWithOrders.XML file
            var customersWithOrders = cb.GetAllCustomers().ToList();

            var xml = new XElement(
                "Northwind",
                from c in customersWithOrders
                select new XElement("customer",
                    new XElement("customerID", c.CustomerID),
                    new XElement("address", c.Address),
                    new XElement("city", c.City),
                    new XElement("companyName", c.CompanyName),
                    new XElement("contactName", c.ContactName),
                    new XElement("contactTitle", c.ContactTitle),
                    new XElement("country", c.Country),
                    new XElement("fax", c.Fax),
                    new XElement("phone", c.Phone),
                    new XElement("postalCode", c.PostalCode),
                    new XElement("region", c.Region),
                    new XElement("orders",
                        from o in c.Orders
                        select new XElement("order",
                            new XElement("orderID", o.OrderID),
                            new XElement("employeeID", o.EmployeeID),
                            new XElement("freight", o.Freight),
                            new XElement("orderDate", o.OrderDate),
                            new XElement("requiredDate", o.RequiredDate),
                            new XElement("shipAddress", o.ShipAddress),
                            new XElement("shipCity", o.ShipCity),
                            new XElement("shipCountry", o.ShipCountry),
                            new XElement("shipName", o.ShipName),
                            new XElement("shippedDate", o.ShippedDate),
                            new XElement("shipPostalCode", o.ShipPostalCode),
                            new XElement("shipRegion", o.ShipRegion),
                            new XElement("shipVia", o.ShipVia)))));

            xml.Save("NorthwindCustomersWithOrders.Xml");
        }

        private static void dumpCustomer(Customer newCustomer)
        {
            Console.WriteLine("CustomerID: {0} - ContactName: {1}",
                newCustomer.CustomerID, newCustomer.ContactName);

            foreach (var o in newCustomer.Orders)
            {
                Console.WriteLine("OrderID: {0} - ShipName: {1}",
                    o.OrderID, o.ShipName);
            }
        }
    }
}
