﻿#define ListOf // Listing 15-11
// #define IEnumerable // Listing 15-17
// #define Queryable

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DomainModel = DevLeap.Linq.Architecture.NTier.DomainModel;
using System.Data.Linq;

namespace DevLeap.Linq.Architecture.NTier.DataLayer.LinqToSql
{
    // Listing 15-10

#if ListOf
    public class CustomerDal : BaseDal<DomainModel.Customer, List<DomainModel.Customer>, String>
#endif
#if IEnumerable
    public class CustomerDal : BaseDal<DomainModel.Customer, IEnumerable<DomainModel.Customer>, String>
#endif
#if Queryable
    public class CustomerDal : BaseDal<DomainModel.Customer, IQueryable<DomainModel.Customer>, String>
#endif
    {
        private static Customer fromDomainModelToLINQ(DomainModel.Customer item)
        {
            // Convert the Domain Model Customer entity into the LINQ to SQL one
            var linqCustomer =
                new Customer
                {
                    CustomerID = item.CustomerID,
                    Address = item.Address,
                    City = item.City,
                    CompanyName = item.CompanyName,
                    ContactName = item.ContactName,
                    ContactTitle = item.ContactTitle,
                    Country = item.Country,
                    Fax = item.Fax,
                    Phone = item.Phone,
                    PostalCode = item.PostalCode,
                    Region = item.Region
                };

            // Converts the list of Domain Model orders
            // into the list of LINQ to SQL orders, using 
            // an ad hoc LINQ query
            linqCustomer.Orders.AddRange(
                from o in item.Orders
                select new Order
                {
                    OrderID = o.OrderID,
                    CustomerID = o.CustomerID,
                    EmployeeID = o.EmployeeID,
                    Freight = o.Freight,
                    OrderDate = o.OrderDate,
                    RequiredDate = o.RequiredDate,
                    ShipAddress = o.ShipAddress,
                    ShipCity = o.ShipCity,
                    ShipCountry = o.ShipCountry,
                    ShipName = o.ShipName,
                    ShippedDate = o.ShippedDate,
                    ShipPostalCode = o.ShipPostalCode,
                    ShipRegion = o.ShipRegion,
                    ShipVia = o.ShipVia
                });
            return linqCustomer;
        }

        public override DomainModel.Customer Add(DomainModel.Customer item)
        {
            // Convert the Domain Model Customer entity into the LINQ to SQL one
            var linqCustomer = fromDomainModelToLINQ(item);

            // Insert the newly created customer into the DataContext
            this.Northwind.Customers.InsertOnSubmit(linqCustomer);

            // Submit the change
            this.Northwind.SubmitChanges();

            // Return the  newly created customer
            return (this.Read(item.CustomerID));
        }

        public override DomainModel.Customer Read(string key)
        {
            // Convert the first LINQ to SQL customer
            // into a Domain Model customer using a 
            // LINQ query expression
            var result =
                (from c in this.Northwind.Customers
                 where c.CustomerID == key
                 select new DomainModel.Customer
                 {
                     CustomerID = c.CustomerID,
                     Address = c.Address,
                     City = c.City,
                     CompanyName = c.CompanyName,
                     ContactName = c.ContactName,
                     ContactTitle = c.ContactTitle,
                     Country = c.Country,
                     Fax = c.Fax,
                     Phone = c.Phone,
                     PostalCode = c.PostalCode,
                     Region = c.Region,
                     // Converts the list of orders of the LINQ to SQL customer
                     // into the list of orders defined in the Domain Model
                     Orders = new List<DevLeap.Linq.Architecture.NTier.DomainModel.Order>(
                         from o in c.Orders
                         select new DomainModel.Order
                         {
                             OrderID = o.OrderID,
                             CustomerID = o.CustomerID,
                             EmployeeID = o.EmployeeID,
                             Freight = o.Freight,
                             OrderDate = o.OrderDate,
                             RequiredDate = o.RequiredDate,
                             ShipAddress = o.ShipAddress,
                             ShipCity = o.ShipCity,
                             ShipCountry = o.ShipCountry,
                             ShipName = o.ShipName,
                             ShippedDate = o.ShippedDate,
                             ShipPostalCode = o.ShipPostalCode,
                             ShipRegion = o.ShipRegion,
                             ShipVia = o.ShipVia
                         }
                         )
                 }).First();

            return (result);
        }

#if ListOf
        public override List<DomainModel.Customer> ReadAll()
#endif
#if IEnumerable
        public override IEnumerable<DomainModel.Customer> ReadAll()
#endif
#if Queryable
        public override IQueryable<DomainModel.Customer> ReadAll()
#endif
        {
            // Convert the LINQ to SQL customers
            // into Domain Model customers using a 
            // LINQ query expression
            var result =
                from c in this.Northwind.Customers
                select new DomainModel.Customer
                {
                    CustomerID = c.CustomerID,
                    Address = c.Address,
                    City = c.City,
                    CompanyName = c.CompanyName,
                    ContactName = c.ContactName,
                    ContactTitle = c.ContactTitle,
                    Country = c.Country,
                    Fax = c.Fax,
                    Phone = c.Phone,
                    PostalCode = c.PostalCode,
                    Region = c.Region,
                    // Converts the list of orders of the LINQ to SQL customer
                    // into the list of orders defined in the Domain Model
                    Orders = new List<DevLeap.Linq.Architecture.NTier.DomainModel.Order>(
                        from o in c.Orders
                        select new DomainModel.Order
                        {
                            OrderID = o.OrderID,
                            CustomerID = o.CustomerID,
                            EmployeeID = o.EmployeeID,
                            Freight = o.Freight,
                            OrderDate = o.OrderDate,
                            RequiredDate = o.RequiredDate,
                            ShipAddress = o.ShipAddress,
                            ShipCity = o.ShipCity,
                            ShipCountry = o.ShipCountry,
                            ShipName = o.ShipName,
                            ShippedDate = o.ShippedDate,
                            ShipPostalCode = o.ShipPostalCode,
                            ShipRegion = o.ShipRegion,
                            ShipVia = o.ShipVia
                        }
                        )
                };

#if ListOf
            return (result).ToList();
#endif
#if IEnumerable
            return (result);
#endif
#if Queryable
            return (result);
#endif
        }

        public override DomainModel.Customer Update(DomainModel.Customer item)
        {
            // Convert the Domain Model Customer entity into the LINQ to SQL one
            var linqCustomer = fromDomainModelToLINQ(item);

            // Attach the modified customer to the DataContext
            this.Northwind.Customers.Attach(linqCustomer, true);
            this.Northwind.Orders.AttachAll(linqCustomer.Orders.Where(o => o.OrderID != 0), true);
            this.Northwind.Orders.InsertAllOnSubmit(linqCustomer.Orders.Where(o => o.OrderID == 0));

            // You should also consider deleted orders, but this is just a small sample
            // not a complete software solution.

            // Submit the change
            this.Northwind.SubmitChanges();

            // Return the  newly created customer
            return (this.Read(item.CustomerID));
        }

        public override bool Delete(DomainModel.Customer item)
        {
            // Convert the Domain Model Customer entity into the LINQ to SQL one
            var linqCustomer = fromDomainModelToLINQ(item);

            // Attach the customer to be deleted to the DataContext
            this.Northwind.Customers.Attach(linqCustomer, false);
            this.Northwind.Orders.AttachAll(linqCustomer.Orders, false);

            // Mark it to be deleted
            this.Northwind.Customers.DeleteOnSubmit(linqCustomer);
            this.Northwind.Orders.DeleteAllOnSubmit(linqCustomer.Orders);

            // Submit the change
            this.Northwind.SubmitChanges();

            // Return the  newly created customer
            return (true);
        }
    }
}
