﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DomainModel = DevLeap.Linq.Architecture.NTier.DomainModel;

namespace DevLeap.Linq.Architecture.NTier.DataLayer.LinqToSql
{
    public class VerboseCustomerDal : BaseDal<DomainModel.Customer, List<DomainModel.Customer>, String>
    {
        public override DomainModel.Customer Add(DomainModel.Customer item)
        {
            // Convert the Domain Model Customer entity into the LINQ to SQL one
            var linqCustomer =
                new Customer
                {
                    CustomerID = item.CustomerID,
                    Address = item.Address,
                    City = item.City,
                    CompanyName = item.CompanyName,
                    ContactName = item.ContactName,
                    ContactTitle = item.ContactTitle,
                    Country = item.Country,
                    Fax = item.Fax,
                    Phone = item.Phone,
                    PostalCode = item.PostalCode,
                    Region = item.Region
                };

            // Converts the list of Domain Model orders
            // into the list of LINQ to SQL orders, using 
            // an iteration over the source orders
            foreach (var o in item.Orders)
            {
                linqCustomer.Orders.Add(new Order
                {
                    OrderID = o.OrderID,
                    CustomerID = o.CustomerID,
                    EmployeeID = o.EmployeeID,
                    Freight = o.Freight,
                    OrderDate = o.OrderDate,
                    RequiredDate = o.RequiredDate,
                    ShipAddress = o.ShipAddress,
                    ShipCity = o.ShipCity,
                    ShipCountry = o.ShipCountry,
                    ShipName = o.ShipName,
                    ShippedDate = o.ShippedDate,
                    ShipPostalCode = o.ShipPostalCode,
                    ShipRegion = o.ShipRegion,
                    ShipVia = o.ShipVia
                });
            }

            // Insert the newly created customer into the DataContext
            this.Northwind.Customers.InsertOnSubmit(linqCustomer);

            // Submit the change
            this.Northwind.SubmitChanges();

            // Return the  newly created customer
            return (this.Read(item.CustomerID));
        }

        // Listing 15-10

        public override DomainModel.Customer Read(string key)
        {
            // Get the first LINQ to SQL customer entity
            // instance with the provided key
            var linqCustomer = this.Northwind.Customers.First(c => c.CustomerID == key);

            // Convert it into a Domain Model customer
            var result = new DomainModel.Customer
                 {
                     CustomerID = linqCustomer.CustomerID,
                     Address = linqCustomer.Address,
                     City = linqCustomer.City,
                     CompanyName = linqCustomer.CompanyName,
                     ContactName = linqCustomer.ContactName,
                     ContactTitle = linqCustomer.ContactTitle,
                     Country = linqCustomer.Country,
                     Fax = linqCustomer.Fax,
                     Phone = linqCustomer.Phone,
                     PostalCode = linqCustomer.PostalCode,
                     Region = linqCustomer.Region
                 };

            result.Orders = new List<DomainModel.Order>();

            // Convert every order obtained from LINQ to SQL 
            // into an Order of the Domain Model
            foreach (var o in linqCustomer.Orders)
            {
                result.Orders.Add(new DomainModel.Order
                {
                    OrderID = o.OrderID,
                    CustomerID = o.CustomerID,
                    EmployeeID = o.EmployeeID,
                    Freight = o.Freight,
                    OrderDate = o.OrderDate,
                    RequiredDate = o.RequiredDate,
                    ShipAddress = o.ShipAddress,
                    ShipCity = o.ShipCity,
                    ShipCountry = o.ShipCountry,
                    ShipName = o.ShipName,
                    ShippedDate = o.ShippedDate,
                    ShipPostalCode = o.ShipPostalCode,
                    ShipRegion = o.ShipRegion,
                    ShipVia = o.ShipVia
                });
            }

            return (result);
        }

        public override List<DomainModel.Customer> ReadAll()
        {
            return base.ReadAll();
        }

        public override DomainModel.Customer Update(DomainModel.Customer item)
        {
            return base.Update(item);
        }

        public override bool Delete(DomainModel.Customer item)
        {
            return base.Delete(item);
        }
    }
}
