﻿#define IEnumerable
// #define Queryable

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DomainModel = DevLeap.Linq.Architecture.NTier.DomainModel;
using System.Xml.Linq;

namespace DevLeap.Linq.Architecture.NTier.DataLayer.LinqToXml
{
#if IEnumerable
    public class CustomerDal : BaseDal<DomainModel.Customer, IEnumerable<DomainModel.Customer>, String>
#endif
#if Queryable
    public class CustomerDal : BaseDal<DomainModel.Customer, IQueryable<DomainModel.Customer>, String>
#endif
    {
        // Listing 15-13

        private static XElement fromDomainModelToXMLNode(DomainModel.Customer item)
        {
            // Convert the Domain Model Customer entity into the XElement customer
            var xmlCustomer =
                new XElement("customer",
                    new XElement("customerID", item.CustomerID),
                    new XElement("address", item.Address),
                    new XElement("city", item.City),
                    new XElement("companyName", item.CompanyName),
                    new XElement("contactName", item.ContactName),
                    new XElement("contactTitle", item.ContactTitle),
                    new XElement("country", item.Country),
                    new XElement("fax", item.Fax),
                    new XElement("phone", item.Phone),
                    new XElement("postalCode", item.PostalCode),
                    new XElement("region", item.Region),
                    new XElement("orders",
                        from o in item.Orders
                        select new XElement("order",
                            new XElement("orderID", o.OrderID),
                            new XElement("employeeID", o.EmployeeID),
                            new XElement("freight", o.Freight),
                            new XElement("orderDate", o.OrderDate),
                            new XElement("requiredDate", o.RequiredDate),
                            new XElement("shipAddress", o.ShipAddress),
                            new XElement("shipCity", o.ShipCity),
                            new XElement("shipCountry", o.ShipCountry),
                            new XElement("shipName", o.ShipName),
                            new XElement("shippedDate", o.ShippedDate),
                            new XElement("shipPostalCode", o.ShipPostalCode),
                            new XElement("shipRegion", o.ShipRegion),
                            new XElement("shipVia", o.ShipVia))));

            return xmlCustomer;
        }

        private static DomainModel.Customer fromXMLNodeToDomainModel(XElement item)
        {
            // Convert the XElement customer to the Domain Model Customer entity
            var customer =
                    new DomainModel.Customer
                    {
                        CustomerID = (String)item.Element("customerID"),
                        Address = (String)item.Element("address"),
                        City = (String)item.Element("city"),
                        CompanyName = (String)item.Element("companyName"),
                        ContactName = (String)item.Element("contactName"),
                        ContactTitle = (String)item.Element("contactTitle"),
                        Country = (String)item.Element("country"),
                        Fax = (String)item.Element("fax"),
                        Phone = (String)item.Element("phone"),
                        PostalCode = (String)item.Element("postalCode"),
                        Region = (String)item.Element("region"),
                        // Converts the list of orders of the LINQ to SQL customer
                        // into the list of orders defined in the Domain Model
                        Orders = new List<DevLeap.Linq.Architecture.NTier.DomainModel.Order>(
                            from ox in item.Element("orders").Elements("order")
                            select new DomainModel.Order
                            {
                                OrderID = (Int32)ox.Element("orderID"),
                                CustomerID = (String)item.Element("customerID"),
                                EmployeeID = (String.IsNullOrEmpty(ox.Element("employeeID").Value) ? null : (Int32?)ox.Element("employeeID")),
                                Freight = (String.IsNullOrEmpty(ox.Element("freight").Value) ? null : (Decimal?)ox.Element("freight")),
                                OrderDate = (String.IsNullOrEmpty(ox.Element("orderDate").Value) ? null : (DateTime?)ox.Element("orderDate")),
                                RequiredDate = (String.IsNullOrEmpty(ox.Element("requiredDate").Value) ? null : (DateTime?)ox.Element("requiredDate")),
                                ShipAddress = (String)ox.Element("shipAddress"),
                                ShipCity = (String)ox.Element("shipCity"),
                                ShipCountry = (String)ox.Element("shipCountry"),
                                ShipName = (String)ox.Element("shipName"),
                                ShippedDate = (String.IsNullOrEmpty(ox.Element("shippedDate").Value) ? null : (DateTime?)ox.Element("shippedDate")),
                                ShipPostalCode = (String)ox.Element("shipPostalCode"),
                                ShipRegion = (String)ox.Element("shipRegion"),
                                ShipVia = (String.IsNullOrEmpty(ox.Element("shipVia").Value) ? null : (Int32?)ox.Element("shipVia")),
                            }
                            )
                    };

            return customer;
        }

        public override DomainModel.Customer Add(DomainModel.Customer item)
        {
            XElement customerNodeToUpdate = this.NorthwindXml.Descendants("customer").FirstOrDefault(x => x.Element("customerID").Value == item.CustomerID);

            // Check for duplicate CustomerID
            if (customerNodeToUpdate != null)
            {
                throw new InvalidOperationException("Customer already exists!");
            }

            XElement xmlCustomer = fromDomainModelToXMLNode(item);
            this.NorthwindXml.Add(xmlCustomer);
            this.Save();

            return (item);
        }

        public override DomainModel.Customer Read(string key)
        {
            var result =
                (from x in this.NorthwindXml.Descendants("customer")
                 where x.Element("customerID").Value == key
                 select new DomainModel.Customer
                  {
                      CustomerID = (String)x.Element("customerID"),
                      Address = (String)x.Element("address"),
                      City = (String)x.Element("city"),
                      CompanyName = (String)x.Element("companyName"),
                      ContactName = (String)x.Element("contactName"),
                      ContactTitle = (String)x.Element("contactTitle"),
                      Country = (String)x.Element("country"),
                      Fax = (String)x.Element("fax"),
                      Phone = (String)x.Element("phone"),
                      PostalCode = (String)x.Element("postalCode"),
                      Region = (String)x.Element("region"),
                      // Converts the list of orders of the LINQ to SQL customer
                      // into the list of orders defined in the Domain Model
                      Orders = new List<DevLeap.Linq.Architecture.NTier.DomainModel.Order>(
                          from ox in x.Element("orders").Elements("order")
                          select new DomainModel.Order
                          {
                              OrderID = (Int32)ox.Element("orderID"),
                              CustomerID = (String)x.Element("customerID"),
                              EmployeeID = (String.IsNullOrEmpty(ox.Element("employeeID").Value) ? null : (Int32?)ox.Element("employeeID")),
                              Freight = (String.IsNullOrEmpty(ox.Element("freight").Value) ? null : (Decimal?)ox.Element("freight")),
                              OrderDate = (String.IsNullOrEmpty(ox.Element("orderDate").Value) ? null : (DateTime?)ox.Element("orderDate")),
                              RequiredDate = (String.IsNullOrEmpty(ox.Element("requiredDate").Value) ? null : (DateTime?)ox.Element("requiredDate")),
                              ShipAddress = (String)ox.Element("shipAddress"),
                              ShipCity = (String)ox.Element("shipCity"),
                              ShipCountry = (String)ox.Element("shipCountry"),
                              ShipName = (String)ox.Element("shipName"),
                              ShippedDate = (String.IsNullOrEmpty(ox.Element("shippedDate").Value) ? null : (DateTime?)ox.Element("shippedDate")),
                              ShipPostalCode = (String)ox.Element("shipPostalCode"),
                              ShipRegion = (String)ox.Element("shipRegion"),
                              ShipVia = (String.IsNullOrEmpty(ox.Element("shipVia").Value) ? null : (Int32?)ox.Element("shipVia")),
                          }
                          )
                  }).First();

            return (result);
        }

#if IEnumerable
        public override IEnumerable<DomainModel.Customer> ReadAll()
#endif
#if Queryable
        public override IQueryable<DomainModel.Customer> ReadAll()
#endif
        {
            var result =
                (from x in this.NorthwindXml.Descendants("customer")
                 select new DomainModel.Customer
                 {
                     CustomerID = (String)x.Element("customerID"),
                     Address = (String)x.Element("address"),
                     City = (String)x.Element("city"),
                     CompanyName = (String)x.Element("companyName"),
                     ContactName = (String)x.Element("contactName"),
                     ContactTitle = (String)x.Element("contactTitle"),
                     Country = (String)x.Element("country"),
                     Fax = (String)x.Element("fax"),
                     Phone = (String)x.Element("phone"),
                     PostalCode = (String)x.Element("postalCode"),
                     Region = (String)x.Element("region"),
                     // Converts the list of orders of the LINQ to SQL customer
                     // into the list of orders defined in the Domain Model
                     Orders = new List<DevLeap.Linq.Architecture.NTier.DomainModel.Order>(
                         from ox in x.Element("orders").Elements("order")
                         select new DomainModel.Order
                         {
                             OrderID = (Int32)ox.Element("orderID"),
                             CustomerID = (String)x.Element("customerID"),
                             EmployeeID = (String.IsNullOrEmpty(ox.Element("employeeID").Value) ? null : (Int32?)ox.Element("employeeID")),
                             Freight = (String.IsNullOrEmpty(ox.Element("freight").Value) ? null : (Decimal?)ox.Element("freight")),
                             OrderDate = (String.IsNullOrEmpty(ox.Element("orderDate").Value) ? null : (DateTime?)ox.Element("orderDate")),
                             RequiredDate = (String.IsNullOrEmpty(ox.Element("requiredDate").Value) ? null : (DateTime?)ox.Element("requiredDate")),
                             ShipAddress = (String)ox.Element("shipAddress"),
                             ShipCity = (String)ox.Element("shipCity"),
                             ShipCountry = (String)ox.Element("shipCountry"),
                             ShipName = (String)ox.Element("shipName"),
                             ShippedDate = (String.IsNullOrEmpty(ox.Element("shippedDate").Value) ? null : (DateTime?)ox.Element("shippedDate")),
                             ShipPostalCode = (String)ox.Element("shipPostalCode"),
                             ShipRegion = (String)ox.Element("shipRegion"),
                             ShipVia = (String.IsNullOrEmpty(ox.Element("shipVia").Value) ? null : (Int32?)ox.Element("shipVia")),
                         }
                         )
                 });

#if IEnumerable
            return (result);
#endif
#if Queryable
            return (result.AsQueryable());
#endif
        }

        public override DomainModel.Customer Update(DomainModel.Customer item)
        {
            XElement xmlCustomer = fromDomainModelToXMLNode(item);

            XElement customerNodeToUpdate = this.NorthwindXml.Descendants("customer").First(x => x.Element("customerID").Value == item.CustomerID);
            customerNodeToUpdate.ReplaceWith(xmlCustomer);
            this.Save();

            return (item);
        }

        public override bool Delete(DomainModel.Customer item)
        {
            XElement customerNodeToDelete = this.NorthwindXml.Descendants("customer").First(x => x.Element("customerID").Value == item.CustomerID);
            customerNodeToDelete.Remove();
            this.Save();

            return (true);
        }
    }
}
