﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevLeap.Linq.Architecture.NTier.DomainModel;

namespace DevLeap.Linq.Architecture.NTier.DataLayer
{
    // Listing 15-7

    public interface ICustomerDal
    {
        /// <summary>
        /// Adds a new customer to the persistence storage
        /// </summary>
        /// <param name="item">The customer instance to add</param>
        /// <returns>The customer that has just been added</returns>
        Customer Add(Customer item);

        /// <summary>
        /// Reads a customer instance from the persistence storage
        /// </summary>
        /// <param name="key">The key to extract the customer instance</param>
        /// <returns>The customer instance, if it exists</returns>
        Customer Read(String key);

        /// <summary>
        /// Reads all the available customer instances from the persistence storage
        /// </summary>
        /// <returns>The list of existing customers</returns>
        /// <remarks>In a real scenario there could be a filtering condition, 
        /// eventually expressed using a LINQ query expression</remarks>
        List<Customer> ReadAll();
        
        /// <summary>
        /// Updates a customer into the persistence storage
        /// </summary>
        /// <param name="item">The customer instance to update</param>
        /// <returns>The customer that has just been updated</returns>
        Customer Update(Customer item);

        /// <summary>
        /// Deletes a customer from the persistence storage
        /// </summary>
        /// <param name="item">The customer to delete</param>
        /// <returns>The result of the deleting operation</returns>
        Boolean Delete(Customer item);
    }
}
