﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevLeap.Linq.Architecture.NTier.DomainModel;

namespace DevLeap.Linq.Architecture.NTier.DataLayer
{
    // Listing 15-8

    public interface IDal<TEntity, TEntityList, TKey>
        where TEntity : BaseEntity, new()
        where TEntityList : IEnumerable<TEntity>
    {
        /// <summary>
        /// Adds a new entity to the persistence storage
        /// </summary>
        /// <param name="item">The entity  instance to add</param>
        /// <returns>The entity  that has just been added</returns>
        TEntity Add(TEntity item);

        /// <summary>
        /// Reads a entity instance from the persistence storage
        /// </summary>
        /// <param name="key">The key to extract the entity instance</param>
        /// <returns>The entity instance, if it exists</returns>
        TEntity Read(TKey key);

        /// <summary>
        /// Reads all the available entity instances from the persistence storage
        /// </summary>
        /// <returns>The list of existing entities</returns>
        /// <remarks>In a real scenario there could be a filtering condition, 
        /// eventually expressed using a LINQ query expression</remarks>
        TEntityList ReadAll();
        
        /// <summary>
        /// Updates a entity into the persistence storage
        /// </summary>
        /// <param name="item">The entity instance to update</param>
        /// <returns>The entity that has just been updated</returns>
        TEntity Update(TEntity item);

        /// <summary>
        /// Deletes an entity from the persistence storage
        /// </summary>
        /// <param name="item">The entity to delete</param>
        /// <returns>The result of the deleting operation</returns>
        Boolean Delete(TEntity item);
    }
}

