using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace DevLeap.Linq.CSharp.CodeFragments {

    // ------------------------------------------------------
    //
    // BEGIN Listing B-4 (part 1)
    //
    public class Writer {
        public string Text;
        public int Counter;
        public void Dump() {
            Console.WriteLine(Text);
            Counter++;
        }
    }
    //
    // END    Listing B-4 (part 1)
    //
    // ------------------------------------------------------

    public class DemoDelegate {
        // ------------------------------------------------------
        //
        // BEGIN Listing B-1
        //
        delegate void SimpleDelegate();
        delegate int ReturnValueDelegate();
        delegate void TwoParamsDelegate(string name, int age);
        //
        // END   Listing B-1
        //
        // ------------------------------------------------------

        // ------------------------------------------------------
        //
        // BEGIN Listing B-2
        //
        void MethodA() { }
        int MethodB() { return 0; }
        void MethodC(string x, int y) { Console.WriteLine("{0}-{1}", x, y); }

        void CreateInstance1() {
            SimpleDelegate a = new SimpleDelegate(MethodA);
            ReturnValueDelegate b = new ReturnValueDelegate(MethodB);
            TwoParamsDelegate c = new TwoParamsDelegate(MethodC);
            // 
            a();
            b();
            c("Marco", 23);
        }
        //
        // END   Listing B-2
        //
        // ------------------------------------------------------


        // ------------------------------------------------------
        //
        // BEGIN Listing B-3
        //
        void CreateInstance2() {
            SimpleDelegate a = MethodA;
            ReturnValueDelegate b = MethodB;
            TwoParamsDelegate c = MethodC;
            // 
            a();
            b();
            c("Marco", 23);
        }
        //
        // END   Listing B-3
        //
        // ------------------------------------------------------

        // ------------------------------------------------------
        //
        // BEGIN Listing B-4 (part 2)
        //
        void Repeat10Times(SimpleDelegate someWork) {
            for (int i = 0; i < 10; i++) someWork();
        }

        void Run1() {
            Writer writer = new Writer();
            writer.Text = "C# chapter";
            this.Repeat10Times(writer.Dump);
            Console.WriteLine(writer.Counter);
        }
        //
        // END   Listing B-4 (part 2)
        //
        // ------------------------------------------------------


        // ------------------------------------------------------
        //
        // BEGIN Listing B-5
        //
        void Run2() {
            int counter = 0;
            this.Repeat10Times(delegate {
                Console.WriteLine("C# chapter");
                counter++;
            });
            Console.WriteLine(counter);
        }
        //
        // END   Listing B-5
        //
        // ------------------------------------------------------

        // ------------------------------------------------------
        //
        // BEGIN Listing B-6
        //
        // **** NOTE: The Repeat10Times method showed in the book is named Worker3 in this sample code
        // ****       to avoid conflict with the Listing B-5 code
        //
        void Worker3(TwoParamsDelegate callback) {
            for (int i = 0; i < 10; i++) callback("Linq book", i);
        }

        void Run3() {
            Worker3(delegate(string text, int age) {
                Console.WriteLine("{0} {1}", text, age);
            });
        }
        //
        // END   Listing B-6
        //
        // ------------------------------------------------------
        
        
        static void Main() {
            // DemoDelegate d = new DemoDelegate();
            // d.CreateInstance2();
            // d.Run3();
            DemoEnumeration.DemoCountdown();
            DemoEnumeration.DemoCountdownForeach();
            DemoEnumeration.DemoCountdownYield();
            DemoEnumeration.DemoCountdownYieldMultiple();
            DemoEnumeration.DemoCountdownYieldTypeSafe();
        }

        static IEnumerator<int> Test() {
            yield return 1;
            yield return 2;
            yield return 3;
        }
    }

    public class DemoEnumeration {
        // ------------------------------------------------------
        //
        // BEGIN Listing B-9
        //
        public static void DemoCountdown() {
            Countdown countdown = new Countdown();
            countdown.StartCountdown = 5;

            IEnumerator i = countdown.GetEnumerator();
            while (i.MoveNext()) {
                int n = (int)i.Current;
                Console.WriteLine(n);
            }
            i.Reset();  // NOT A GOOD PRACTICE - SEE BOOK TEXT
            while (i.MoveNext()) {
                int n = (int)i.Current;
                Console.WriteLine("{0} BIS", n);
            }
        }
        //
        // END    Listing B-9
        //
        // ------------------------------------------------------


        // ------------------------------------------------------
        //
        // BEGIN Listing B-10
        //
        public static void DemoCountdownForeach() {
            Countdown countdown = new Countdown();
            countdown.StartCountdown = 5;

            foreach (int n in countdown) {
                Console.WriteLine(n);
            }
            foreach (int n in countdown) {
                Console.WriteLine("{0} BIS", n);
            }
        }
        //
        // END    Listing B-10
        //
        // ------------------------------------------------------

        public static void DemoCountdownYield() {
            CountdownYield countdown = new CountdownYield();
            countdown.StartCountdown = 5;

            foreach (int n in countdown) {
                Console.WriteLine(n);
            }
            foreach (int n in countdown) {
                Console.WriteLine("{0} BIS", n);
            }
        }
        public static void DemoCountdownYieldMultiple() {
            CountdownYieldMultiple countdown = new CountdownYieldMultiple();

            foreach (int n in countdown) {
                Console.WriteLine(n);
            }
            foreach (int n in countdown) {
                Console.WriteLine("{0} BIS", n);
            }
        }
        public static void DemoCountdownYieldTypeSafe() {
            CountdownYieldTypeSafe countdown = new CountdownYieldTypeSafe();
            countdown.StartCountdown = 5;

            foreach (int n in countdown) {
                Console.WriteLine(n);
            }
            foreach (int n in countdown) {
                Console.WriteLine("{0} BIS", n);
            }
        }
    }


    // ------------------------------------------------------
    //
    // BEGIN Listing B-8
    //
    public class Countdown : IEnumerable {
        public int StartCountdown;

        public IEnumerator GetEnumerator() {
            return new CountdownEnumerator(this);
        }
    }

    public class CountdownEnumerator : IEnumerator {
        private int _counter;
        private Countdown _countdown;

        public CountdownEnumerator(Countdown countdown) {
            _countdown = countdown;
            Reset();
        }

        public bool MoveNext() {
            if (_counter > 0) {
                _counter--;
                return true;
            }
            else {
                return false;
            }
        }

        public void Reset() {
            _counter = _countdown.StartCountdown;
        }

        public object Current {
            get {
                return _counter;
            }
        }
    }
    //
    // END    Listing B-8
    //
    // ------------------------------------------------------

    // ------------------------------------------------------
    //
    // BEGIN Listing B-11
    //
    public class CountdownYield : IEnumerable {
        public int StartCountdown;

        public IEnumerator GetEnumerator() {
            for (int i = StartCountdown - 1; i >= 0; i--) {
                yield return i;
            }
        }
    }
    //
    // END    Listing B-11
    //
    // ------------------------------------------------------

    // ------------------------------------------------------
    //
    // BEGIN Listing B-14
    //
    public class CountdownYieldMultiple : IEnumerable {
        public IEnumerator GetEnumerator() {
            yield return 4;
            yield return 3;
            yield return 2;
            yield return 1;
            yield return 0;
        }
    }
    //
    // END    Listing B-14
    //
    // ------------------------------------------------------

    // ------------------------------------------------------
    //
    // BEGIN Listing B-15
    //
    public class CountdownYieldTypeSafe : IEnumerable<int> {
        public int StartCountdown;

        IEnumerator IEnumerable.GetEnumerator() {
            return this.GetEnumerator();
        }
        public IEnumerator<int> GetEnumerator() {
            for (int i = StartCountdown - 1; i >= 0; i--) {
                yield return i;
            }
        }
    }
    //
    // END    Listing B-15
    //
    // ------------------------------------------------------

}