﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Globalization;

namespace DevLeap.Linq.CSharp.ExtensionMethods {

    static class ExtensionMethods {
        // ------------------------------------------------------
        //
        // BEGIN Listing B-22
        //
        public static void Demo() {
            decimal x = 1234.568M;
            Console.WriteLine(x.FormattedUS());
            Console.WriteLine(x.FormattedIT());
            Console.WriteLine(FormattedUS(x));
            Console.WriteLine(FormattedIT(x));
        }

        static CultureInfo formatUS = new CultureInfo("en-US");
        static CultureInfo formatIT = new CultureInfo("it-IT");
        public static string FormattedUS(this decimal d) {
            return String.Format(formatIT, "{0:#,0.00}", d);
        }

        public static string FormattedIT(this decimal d) {
            return String.Format(formatUS, "{0:#,0.00}", d);
        }
        //
        // END    Listing B-22
        //
        // ------------------------------------------------------

        // ------------------------------------------------------
        //
        // BEGIN Listing B-23
        //
        public static decimal Double(this decimal d) {
            return d + d;
        }
        public static decimal Triple(this decimal d) {
            return d * 3;
        }
        public static decimal Increase(this decimal d) {
            return d + 1;
        }
        public static decimal Decrease(this decimal d) {
            return d - 1;
        }
        public static decimal Half(this decimal d) {
            return d / 2;
        }
        //
        // END    Listing B-23
        //
        // ------------------------------------------------------


        // ------------------------------------------------------
        //
        // BEGIN Listing B-24
        //
        public static void Listing2_24() {
            decimal x = 14M, y = 14M;
            x = Half(Triple(Decrease(Decrease(Double(Increase(x))))));
            y = y.Increase().Double().Decrease().Decrease().Triple().Half();

            Console.WriteLine(x);
            Console.WriteLine(y);
        }
        //
        // END    Listing B-24
        //
        // ------------------------------------------------------


    }


    static class Traditional {
        public static void Demo() {
            decimal x = 1234.568M;
            Console.WriteLine(FormattedUS(x));
            Console.WriteLine(FormattedIT(x));
        }

        static CultureInfo formatUS = new CultureInfo("en-US");
        static CultureInfo formatIT = new CultureInfo("it-IT");
        public static string FormattedUS(decimal d) {
            return String.Format(formatIT, "{0:#,0.00}", d);
        }

        public static string FormattedIT(decimal d) {
            return String.Format(formatUS, "{0:#,0.00}", d);
        }
    }

    // ------------------------------------------------------
    //
    // BEGIN Listing B-25
    //
    static class Visualizer {
        public static void Display(this object o) {
            string s = o.ToString();
            Console.WriteLine(s);
        }
        public static void Display(this Customer c) {
            string s = String.Format("Name={0}", c.Name);
            Console.WriteLine(s);
        }
    }
    //
    // END    Listing B-25
    //
    // ------------------------------------------------------

    // ------------------------------------------------------
    //
    // BEGIN Listing B-26
    //
    public class Customer {
        protected int Id;
        public string Name;

        public Customer(int id) {
            this.Id = id;
        }
        /*
            // Uncomment this method to see the call to Display specific version
            public void Display() {
                string s = String.Format( "{0}-{1}", Id, Name );
                Console.WriteLine( s );
            }
        */
    }
    //
    // END    Listing B-26
    //
    // ------------------------------------------------------


    // ------------------------------------------------------
    //
    // BEGIN Listing B-27
    //
    public class A {
        public virtual void X() {
            Console.WriteLine("A.X()");
        }
    }
    public class B : A {
        public override void X() {
            Console.WriteLine("B.X()");
        }
        public void Y() {
            Console.WriteLine("B.Y()");
        }
    }
    static public class E {
        static void X(this A a) {
            Console.WriteLine("E.X()");
        }
        static void Y(this A b) {
            Console.WriteLine("E.Y()");
        }
        public static void Demo() {
            A a = new A();
            B b = new B();
            A c = new B();

            a.X(); // Call A.X
            b.X(); // Call B.X
            c.X(); // Call B.X

            a.Y(); // Call E.Y
            b.Y(); // Call B.Y
            c.Y(); // Call E.Y
        }
    }
    //
    // END    Listing B-27
    //
    // ------------------------------------------------------

    static class Program {
        static void Main() {
            Traditional.Demo();
            ExtensionMethods.Demo();
            ExtensionMethods.Listing2_24();

            Customer c = new Customer(1);
            c.Name = "Marco";
            c.Display();

            E.Demo();

            LambdaUseExtension.Demo();

            Book.Demo.Ext.Sample.DemoCall();
        }
    }

    // Extension method resolution - this example is not included in the book
    namespace Book {
        static class Utilities {
            public static string MyUpper(this string s) {
                return s.ToUpper();
            }

        }
    }

    namespace Book.Demo.Ext {
        public static class Sample {
            public static void DemoCall() {
                string s = "Marco";
                Console.WriteLine(s.MyUpper());
            }
        }
    }

}