﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Collections;

namespace DevLeap.Linq.CSharp.ObjectInitializers {
    public class Customer {
        public int Age;
        public string Name;
        public string Country;
        public Customer(string name, int age) {
            this.Name = name;
            this.Age = age;
        }
        public Customer() {
            Age = -1;
            Name = "<Unknown>";
            Country = "<Unknown>";
        }
    }
    // ------------------------------------------------------
    //
    // BEGIN Listing B-32 (part 1)
    //

    public class Point {
        public int X { get; set; }
        public int Y { get; set; }
    }

    public class Rectangle {
        public Point TL { get; set; }
        public Point BR { get; set; }
    }
    //
    // END    Listing B-32 (part 1)
    //
    // ------------------------------------------------------


    static class ObjectInitializers {
        static void Main() {
            DemoInit();
            ClassicalInitialization();
            NewSyntax();
            Nested();
            ExplicitConstructors();
            Collections();
        }
        static void DemoInit() {
            int i = 3;
            string name = "Unknown'";
            Customer customer = new Customer("Tom", 32);
        }
        // ------------------------------------------------------
        //
        // BEGIN Listing B-29
        //
        static void ClassicalInitialization() {
            Customer customer = new Customer();
            customer.Name = "Marco";
            customer.Country = "Italy";
        }
        //
        // END    Listing B-29
        //
        // ------------------------------------------------------


        // ------------------------------------------------------
        //
        // BEGIN Listing B-30
        //
        static void NewSyntax() {
            Customer customer = new Customer() { Name = "Marco", Country = "Italy" };
        }
        //
        // END    Listing B-30
        //
        // ------------------------------------------------------

        // ------------------------------------------------------
        //
        // BEGIN Listing B-31
        //
        static void ExplicitConstructors() {
            Customer c1 = new Customer() { Name = "Marco", Country = "Italy" };
            Customer c2 = new Customer("Paolo", 21) { Country = "Italy" };
            Customer c3 = new Customer { Name = c1.Name, Country = c2.Country, Age = c2.Age };
        }
        //
        // END    Listing B-31
        //
        // ------------------------------------------------------

        // ------------------------------------------------------
        //
        // BEGIN Listing B-32 (part 2)
        //

        static void Nested() {
            Rectangle r = new Rectangle {
                              TL = new Point { X = 0, Y = 1 },
                              BR = new Point { X = 2, Y = 3 }
                          };
        }
        //
        // END    Listing B-32 (part 2)
        //
        // ------------------------------------------------------
        static void Arrays() {
            int[] integers = { 1, 3, 9, 18 };
            string[] customers = { "Jack", "Paolo", "Marco" };
        }
        // ------------------------------------------------------
        //
        // BEGIN Listing B-34
        //
        static void Collections() {
            List<int> integers1 = new List<int> { 1, 3, 9, 18 };
            List<Customer> list1 = new List<Customer> {
                new Customer( "Jack", 28 ) { Country = "USA"},
                new Customer { Name = "Paolo" },
                new Customer { Name = "Marco", Country = "Italy" },
            };

            foreach (Customer c in list1) {
                Console.WriteLine(c.Name);
            }

            Dictionary<int, Customer> customers;
            customers = new Dictionary<int, Customer> {
                { 1, new Customer( "Jack", 28 ) { Country = "USA"} },
                { 2, new Customer { Name = "Paolo" } },
                { 3, new Customer { Name = "Marco", Country = "Italy" } }
            };

            foreach (var c in customers) {
                Console.WriteLine("{0}-{1}", c.Key, c.Value.Name);
            }

        }
        static void LegacyCollections() {
            ArrayList integers2 = new ArrayList() { 1, 3, 9, 18 };
            ArrayList list2 = new ArrayList {
                new Customer( "Jack", 28 ) { Country = "USA"},
                new Customer { Name = "Paolo" },
                new Customer { Name = "Marco", Country = "Italy" },
            };

            foreach (Customer c in list2) {
                Console.WriteLine(c.Name);
            }
        }
        //
        // END    Listing B-34
        //
        // ------------------------------------------------------

        // ------------------------------------------------------
        //
        // BEGIN Listing B-33 (part 2)
        //
        static void Owned() {
            OwnedObjects.Rectangle r = new OwnedObjects.Rectangle {
                                           TL = { X = 0, Y = 1 },
                                           BR = { X = 2, Y = 3 }
                                       };
        }
        //
        // END    Listing B-33 (part 2)
        //
        // ------------------------------------------------------
    }


    namespace OwnedObjects {
        // ------------------------------------------------------
        //
        // BEGIN Listing B-33 (part 1)
        //
        public class Rectangle {
            Point tl = new Point();
            Point br = new Point();
            public Point TL { get { return tl; } }
            public Point BR { get { return br; } }
        }
        //
        // END    Listing B-33 (part 1)
        //
        // ------------------------------------------------------

    }
}