﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DevLeap.FlightStatus {
    // 
    // Listing 12-24
    // 

    /// <summary>
    /// Flight Status Service
    /// Please note that FlightStatus.QueryFilter is defined into 
    /// another file using the partial class syntax
    /// </summary>
    public partial class FlightStatusService {

        public List<Flight> FlightSearch(QueryFilter pars) {
            return FlightSearch(pars, -1);
        }

        /// <summary>
        /// API to query flights status - parameters are constraints to filter only desired flights
        /// </summary>
        /// <param name="pars">Parameters for flights status search</param>
        /// <param name="maxFlights">Maximum numbers of flights in the result 
        /// (-1 for all, 0 for empty result)</param>
        /// <returns>List of flights matching search parameters</returns>
        public List<Flight> FlightSearch(QueryFilter pars, int maxFlights) {
            Console.WriteLine("---- FlightQuery execution ----");
            if (maxFlights >= 0) Console.WriteLine("Maximum returned flights: {0}", maxFlights);
            Console.WriteLine(pars);

            List<Flight> result = new List<Flight>();
            foreach (var f in flights) {
                if ((maxFlights >= 0) && (result.Count >= maxFlights)) {
                    return result;
                }
                if ((pars.Airline != null) && (pars.Airline != f.Airline)) continue;
                if ((pars.FlightNumber != null) && (pars.FlightNumber != f.FlightNumber)) continue;
                if ((pars.Airline != null) && (pars.Airline != f.Airline)) continue;
                if ((pars.DepartureAirport != null) && (pars.DepartureAirport != f.Departure.Airport)) continue;
                if ((pars.ArrivalAirport != null) && (pars.ArrivalAirport != f.Arrival.Airport)) continue;
                if ((pars.MinMinutesToArrival >= 0) && (pars.MinMinutesToArrival > f.TimeToArrival.TotalMinutes)) continue;
                if ((pars.MaxMinutesToArrival >= 0) && (pars.MaxMinutesToArrival < f.TimeToArrival.TotalMinutes)) continue;
                if ((pars.MinGroundSpeed >= 0) && (pars.MinGroundSpeed > f.GroundSpeed)) continue;
                if ((pars.MaxGroundSpeed >= 0) && (pars.MaxGroundSpeed < f.GroundSpeed)) continue;
                if ((pars.MinAltitude >= 0) && (pars.MinAltitude > f.Altitude)) continue;
                if ((pars.MaxAltitude >= 0) && (pars.MaxAltitude < f.Altitude)) continue;
                result.Add(f);
            }
            return result;
        }

        static private Flight[] flights = {
            new Flight { Airline = "B6", FlightNumber = "1", Aircraft = "Airbus A320", 
                  Departure = new AirportInformation { Airport = "JFK", ScheduledTime = new DateTime(2007,12,25,10,55,0), ActualTime = new DateTime(2007,12,25,14,47,0) }, 
                  Arrival   = new AirportInformation { Airport = "SJD", ScheduledTime = new DateTime(2007,12,25,13,50,0), ActualTime = new DateTime(2007,12,25,17,31,0) }, 
                  TimeToArrival = new TimeSpan(1,49,0),
                  GroundSpeed = 391, 
                  Altitude = 34000 },
            new Flight { Airline = "NW", FlightNumber = "1", Aircraft = "Boeing 747-400", 
                  Departure = new AirportInformation { Airport = "LAX", ScheduledTime = new DateTime(2007,12,25,11,30,0), ActualTime = new DateTime(2007,12,25,16,20,0) }, 
                  Arrival   = new AirportInformation { Airport = "NRT", ScheduledTime = new DateTime(2007,12,26,16,20,0), ActualTime = new DateTime(2007,12,26,16,38,0) }, 
                  TimeToArrival = new TimeSpan(10,55,0),
                  GroundSpeed = 419, 
                  Altitude = 34000 },
            new Flight { Airline = "WN", FlightNumber = "1002", Aircraft = "Boeing 737-700", 
                  Departure = new AirportInformation { Airport = "RNO", ScheduledTime = new DateTime(2007,12,25,12,15,0), ActualTime = new DateTime(2007,12,25,12,24,0) }, 
                  Arrival   = new AirportInformation { Airport = "SJC", ScheduledTime = new DateTime(2007,12,25,13,15,0), ActualTime = new DateTime(2007,12,25,13,02,0) }, 
                  TimeToArrival = new TimeSpan(0,18,0),
                  GroundSpeed = 447, 
                  Altitude = 22000 },
            new Flight { Airline = "AC", FlightNumber = "1", Aircraft = "Airbus A340-300", 
                  Departure = new AirportInformation { Airport = "YYZ", ScheduledTime = new DateTime(2007,12,25,13,55,0), ActualTime = new DateTime(2007,12,25,14,14,0) }, 
                  Arrival   = new AirportInformation { Airport = "NRT", ScheduledTime = new DateTime(2007,12,26,17,35,0), ActualTime = new DateTime(2007,12,26,17,11,0) }, 
                  TimeToArrival = new TimeSpan(1,49,0),
                  GroundSpeed = 424, 
                  Altitude = 30000 },
            new Flight { Airline = "NK", FlightNumber = "117", Aircraft = "Airbus A319", 
                  Departure = new AirportInformation { Airport = "DTW", ScheduledTime = new DateTime(2007,12,25,13,09,0), ActualTime = new DateTime(2007,12,25,13,48,0) }, 
                  Arrival   = new AirportInformation { Airport = "MCO", ScheduledTime = new DateTime(2007,12,25,15,43,0), ActualTime = new DateTime(2007,12,25,16,01,0) }, 
                  TimeToArrival = new TimeSpan(0,12,0),
                  GroundSpeed = 359, 
                  Altitude = 22400 },
            new Flight { Airline = "DL", FlightNumber = "1", Aircraft = "Boeing 757-200", 
                  Departure = new AirportInformation { Airport = "SFO", ScheduledTime = new DateTime(2007,12,25,09,00,0), ActualTime = new DateTime(2007,12,25,09,08,0) }, 
                  Arrival   = new AirportInformation { Airport = "JFK", ScheduledTime = new DateTime(2007,12,25,17,50,0), ActualTime = new DateTime(2007,12,25,16,55,0) }, 
                  TimeToArrival = new TimeSpan(1,06,0),
                  GroundSpeed = 482, 
                  Altitude = 37000 },
            new Flight { Airline = "TZ", FlightNumber = "1040", Aircraft = "Boeing 737-700", 
                  Departure = new AirportInformation { Airport = "OAK", ScheduledTime = new DateTime(2007,12,25,12,05,0), ActualTime = new DateTime(2007,12,25,12,29,0) }, 
                  Arrival   = new AirportInformation { Airport = "MDW", ScheduledTime = new DateTime(2007,12,25,18,10,0), ActualTime = new DateTime(2007,12,25,17,59,0) }, 
                  TimeToArrival = new TimeSpan(3,09,0),
                  GroundSpeed = 484, 
                  Altitude = 34900 },
            new Flight { Airline = "UA", FlightNumber = "1", Aircraft = "Boeing 777-200", 
                  Departure = new AirportInformation { Airport = "ORD", ScheduledTime = new DateTime(2007,12,25,10,10,0), ActualTime = new DateTime(2007,12,25,10,10,0) }, 
                  Arrival   = new AirportInformation { Airport = "HNL", ScheduledTime = new DateTime(2007,12,25,15,22,0), ActualTime = new DateTime(2007,12,25,15,07,0) }, 
                  TimeToArrival = new TimeSpan(4,15,0),
                  GroundSpeed = 411, 
                  Altitude = 36000 },

        };
    }
}
