﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using DevLeap.FlightStatus;
using MattWarren.ExpressionsTools;

namespace DevLeap.Linq.FlightStatus {
    // 
    // Listing 12-31
    // 
    public class FlightQueryProvider : BaseQueryProvider {
        // Force developers to pass a FlightStatusService in the constructor
        protected FlightQueryProvider() {}

        private FlightStatusService flightStatus;

        public FlightQueryProvider(FlightStatusService flightStatus) {
            if (flightStatus == null) {
                throw new ArgumentNullException("flightStatus");
            }
            this.flightStatus = flightStatus;
        }

        #region BaseQueryProvider abstract methods implementation
        public override object Execute(Expression expression) {
            FlightQueryParameters parameters = this.Translate(expression);
            return flightStatus.FlightSearch(parameters.Filter, parameters.MaxFlights);
        }

        public override IQueryable CreateQuery(Expression expression) {
            return new FlightQuery(this, expression);
        }
        #endregion BaseQueryProvider abstract methods implementation

        public string GetQueryText(Expression expression) {
            FlightQueryParameters parameters = this.Translate(expression);
            return parameters.ToString();
        }

        private FlightQueryParameters Translate(Expression expression) {
            expression = Evaluator.PartialEval(expression);
            return new FlightQueryTranslator().Translate(expression);
        }
    }
}
