﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevLeap.FlightStatus;
using DevLeap.Linq.FlightStatus;

namespace DevLeap.Linq.ExtendingLINQ.FlightStatus {
    class Program {
        static void Main(string[] args) {
            Console.WriteLine("**** SearchFlightsOldWay ****");
            SearchFlightsOldWay();
            Console.WriteLine("****  SearchFlightsCS30  ****");
            SearchFlightsCS30();
            Console.WriteLine("****  SearchFlightsLinq  ****");
            SearchFlightsLinq();

            OtherLinqQueriesFlightsLinq();
        }

        // 
        // Listing 12-27
        // 
        /// <summary>
        /// Search flights in the old way
        /// </summary>
        static void SearchFlightsOldWay() {
            FlightStatusService flightStatus = new FlightStatusService();

            FlightStatusService.QueryFilter filterTimeSpeed = new FlightStatusService.QueryFilter();
            filterTimeSpeed.MaxGroundSpeed = 400;
            filterTimeSpeed.MaxMinutesToArrival = 30;

            List<Flight> flightsNearLanding = flightStatus.FlightSearch(filterTimeSpeed);
            Dump(flightsNearLanding);

            FlightStatusService.QueryFilter filterAltitudeAirline = new FlightStatusService.QueryFilter();
            filterAltitudeAirline.MinAltitude = 20000;
            filterAltitudeAirline.MaxAltitude = 30000;
            filterAltitudeAirline.Airline = "WN";

            List<Flight> flightsMediumAltitude = flightStatus.FlightSearch(filterAltitudeAirline);
            Dump(flightsMediumAltitude);

            string airportCode = "NRT"; // Tokyo 
            FlightStatusService.QueryFilter filterTokyo = new FlightStatusService.QueryFilter();
            filterTokyo.ArrivalAirport = airportCode;

            // Filter only 1 output rows
            List<Flight> flightsTokyo = flightStatus.FlightSearch(filterAltitudeAirline, 1);
            Dump(flightsMediumAltitude);
        }

        // 
        // Listing 12-28
        // 
        /// <summary>
        /// Search flights with C# 3.0
        /// </summary>
        static void SearchFlightsCS30() {
            FlightStatusService flightStatus = new FlightStatusService();

            var flightsNearLanding =
                flightStatus.FlightSearch(
                    new FlightStatusService.QueryFilter {
                        MaxGroundSpeed = 400,
                        MaxMinutesToArrival = 30
                    });
            Dump(flightsNearLanding);

            var flightsMediumAltitude =
                flightStatus.FlightSearch(
                    new FlightStatusService.QueryFilter {
                        MinAltitude = 20000,
                        MaxAltitude = 30000,
                        Airline = "WN"
                    });
            Dump(flightsMediumAltitude);

            string airportCode = "NRT"; // Tokyo
            var flightsTokyo =
                flightStatus.FlightSearch(
                    new FlightStatusService.QueryFilter {
                        ArrivalAirport = airportCode
                    },
                    1); // Filter only 1 output rows
            Dump(flightsTokyo);
        }

        // 
        // Listing 12-29
        // 
        /// <summary>
        /// Search flights with LINQ
        /// </summary>
        static void SearchFlightsLinq() {
            FlightStatusService flightStatus = new FlightStatusService();

            var flightsNearLanding =
                from   f in flightStatus.AsQueryable()
                where  f.GroundSpeed <= 400
                       && f.TimeToArrival.TotalMinutes <= 30
                select f;
            Dump(flightsNearLanding);

            var flightsMediumAltitude =
                from f in flightStatus.AsQueryable()
                where f.GroundSpeed <= 400
                       && f.TimeToArrival.TotalMinutes <= 30
                select f;
            Dump(flightsMediumAltitude);

            string airportCode = "NRT"; // Tokyo
            var filterTokyo =
                (from f in flightStatus.AsQueryable()
                 where f.Arrival.Airport == airportCode
                 select f)
                .Take(1); // Filter only 1 output rows
            Dump(filterTokyo);
            Console.WriteLine(filterTokyo.ToString());
        }

        private static void Dump(IEnumerable<Flight> flights) {
            foreach (var f in flights) {
                Console.WriteLine(f);
                Console.WriteLine("--------------------");
            }
        }

        // 
        // Listing 12-41
        // 
        static void OtherLinqQueriesFlightsLinq() {
            FlightStatusService flightStatus = new FlightStatusService();

            var query1 =
                from f in flightStatus.AsQueryable()
                where f.Airline == "UA" && f.Altitude >= 10000
                select f;
            Console.WriteLine("** query1 **");
            Console.WriteLine(query1);
            // Dump(query1);

            var query2 =
                (from f in flightStatus.AsQueryable()
                 where f.Arrival.Airport == "NRT" 
                       && f.TimeToArrival.TotalHours >= 2
                 select f).Take(2);
            Console.WriteLine("** query2 **");
            Console.WriteLine(query2);
            //            Dump(query2);
        }

    }
}
