﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DevLeap.Linq.ExtendingLINQ.OnlyVisible {
    static class Extensions {
        // 
        // Listing 12-11
        // 
        public static IEnumerable<TSource> OnlyVisible<TSource>(
                this IEnumerable<TSource> source)
            where TSource : IVisible {

            foreach (TSource item in source) {
                if (item.Visible) {
                    yield return item;
                }
            }
        }
    }

    class Program {
        static void Main(string[] args) {
            // 
            // Listing 12-12
            // 
            Customer[] customers = 
                { new Customer { Name = "John", Age = 24, Visible = false }, 
                  new Customer { Name = "Allison", Age = 45 }, 
                  new Customer { Name = "Brad", Age = 33 } };

            var query =
                from c in customers.OnlyVisible()
                where c.Age < 40
                select c;

            foreach (var row in query) {
                Console.WriteLine(row);
            }

        }
    }
}
