﻿// 
// The code in this listing has been written Jomo Fisher on his blog:
/// - http://blogs.msdn.com/jomo_fisher/archive/2007/05/07/visitor-revisitted-linq-function-composablity-and-chain-of-responsibility.aspx
// and has been released under the Microsoft Public License published below
// 

/*
Microsoft Public License (Ms-PL)
Published: October 12, 2006
  
Related Links
• Shared Source Licenses 
• Microsoft Limited Public License 
• Microsoft Reciprocal License 
• Microsoft Reference License 
  
 
This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

1. Definitions
The terms “reproduce,” “reproduction,” “derivative works,” and “distribution” have the same meaning here as under U.S. copyright law.

A “contribution” is the original software, or any additions or changes to the software.

A “contributor” is any person that distributes its contribution under this license.

“Licensed patents” are a contributor’s patent claims that read directly on its contribution.

Top of page
2. Grant of Rights
(A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.

(B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

Top of page
3. Conditions and Limitations
(A) No Trademark License- This license does not grant you rights to use any contributors’ name, logo, or trademarks.

(B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.

(C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.

(D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.

(E) The software is licensed “as-is.” You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DevLeap.Linq.ExtendingLINQ.PredicateAnalysis {
    /// <summary>
    /// This class was originally published by Jomo Fisher on his blog:
    /// - http://blogs.msdn.com/jomo_fisher/archive/2007/05/07/visitor-revisitted-linq-function-composablity-and-chain-of-responsibility.aspx
    /// </summary>
    static class FuncOp {
        private class Binding<T, TResult> {
            public Func<T, TResult> Bound { get; private set; }
            public Func<Func<T, TResult>, Func<T, TResult>, T, TResult> Unbound { get; private set; }
            public Binding<T, TResult> LastBinding { get; private set; }
            public Binding(Func<T, TResult> bound, Func<Func<T, TResult>, Func<T, TResult>, T, TResult> unbound, Binding<T, TResult> lastBinding) {
                Bound = bound;
                Unbound = unbound;
                LastBinding = lastBinding;
            }
            public TResult Invoke(T t) { return Bound(t); }
        }
        private class RefOf<T> {
            public T Ref { get; set; }
        }

        public static Func<T, TResult> Create<T, TResult>(
            Func<Func<T, TResult>, T, TResult> function) {

            var selfRef = new RefOf<Func<T, TResult>>();
            var unbound = new Func<Func<T, TResult>, Func<T, TResult>, T, TResult>(
                (s, l, t) => function(s, t)
            );
            selfRef.Ref = Chain(selfRef, unbound, null);
            return selfRef.Ref;
        }
        private static Func<T, TResult> Chain<T, TResult>(
            RefOf<Func<T, TResult>> selfRef, 
            Func<Func<T, TResult>, Func<T, TResult>, T, TResult> unbound, 
            Binding<T, TResult> lastBinding) {
            Func<T, TResult> last = null;
            Func<T, TResult> bound = (t) => unbound(selfRef.Ref, last, t);
            if (lastBinding != null)
                last = Chain(selfRef, lastBinding.Unbound, lastBinding.LastBinding);
            var binding = new Binding<T, TResult>(bound, unbound, lastBinding);
            return new Func<T, TResult>(binding.Invoke);
        }
        public static Func<T, TResult> Chain<T, TResult>(
            this Func<T, TResult> lastFunction, 
            Func<Func<T, TResult>, Func<T, TResult>, T, TResult> selfFunction) {

            var selfRef = new RefOf<Func<T, TResult>>();
            var lastBinding = (Binding<T, TResult>) lastFunction.Target;
            selfRef.Ref = Chain<T, TResult>(selfRef, selfFunction, lastBinding);
            return selfRef.Ref;
        }
    }
}
