﻿using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.Reflection;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;

namespace DevLeap.Linq.ParallelLinq {
    class Program {
        static void Main(string[] args) {
            // First run (cold cache)
            //DemoFor();
            //DemoForEach();
            //DemoDo();
            //DemoTask();
            //DemoFuture();
            //// Second run (warm cache)
            //DemoFor();
            //DemoForEach();
            //DemoDo();
            //DemoTask();
            //DemoFuture();
            //DemoConcurrency_RaceCondition();
            //DemoConcurrency_Safe();
            StandardLinq();
            SamplePLinq();
            SamplePLinq_StopAndGo();
            SamplePLinq_Inverted();
            SamplePLinq_OrderBy();
            SamplePLinq_PreserveOrdering();
            SamplePLinq_RaceCondition();
            SamplePLinq_LoadList_Safe();
            SamplePLinq_LoadList_Unsafe();
            SamplePLinq_ToList();
            SamplePLinq_ExceptionHandling();
            XmlPLinq();
            SqlPLinq();

            StandardLinq();
            SamplePLinq();
            SamplePLinq_StopAndGo();
            SamplePLinq_Inverted();
            SamplePLinq_OrderBy();
            SamplePLinq_PreserveOrdering();
            SamplePLinq_RaceCondition();
            SamplePLinq_LoadList_Safe();
            SamplePLinq_LoadList_Unsafe();
            SamplePLinq_ToList();
            SamplePLinq_ExceptionHandling();
            XmlPLinq();
            SqlPLinq();
            // ReflectionSearch();
            //CompareReflectionScan();
        }

        static void DemoFor() {
            SequentialFor();
            ParallelFor();
        }
        static int[] LoadData(int size) {
            int[] data = new int[size];
            for (int i = 0; i < data.Length; i++) {
                data[i] = i;
            }
            return data;
        }
        static void DemoForEach() {
            int[] data = LoadData(100);
            SequentialForEach(data);
            ParallelForEach(data);
        }
        static void DemoDo() {
            DemoDo_Sequential();
            DemoDo_Parallel();
        }
        static void ProcessItem(ref long value) {
            value += 1;
            value -= 1;
        }
        static void ProcessData(int index) {
            // Simulate some data processing here
            for (long i = 0; i < 3000000 + index * 1000; i++ ) {
                ProcessItem(ref i);
            }
        }
        static void DataCalculation(int index) {
            // Simulate some data processing here
            for (long i = 0; i < 3000000 / (index + 1); i++ ) {
                ProcessItem(ref i);
            }
        }

        //
        // Listing 13-1
        //
        static void SequentialFor() {
            Stopwatch sw = Stopwatch.StartNew();
            for (int index = 0; index < 100; index++) {
                ProcessData(index);
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Sequential for: {0} milliseconds", elapsed);
        }

        static void ParallelFor() {
            Stopwatch sw = Stopwatch.StartNew();
            Parallel.For(0, 100, (index) => {
                ProcessData(index);
            });
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Parallel for: {0} milliseconds", elapsed);
        }

        //
        // Listing 13-2
        //
        static void SequentialForEach(int[] data) {
            Stopwatch sw = Stopwatch.StartNew();
            foreach (int value in data) {
                ProcessData(value);
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Sequential foreach: {0} milliseconds", elapsed);
        }

        static void ParallelForEach(int[] data) {
            Stopwatch sw = Stopwatch.StartNew();
            Parallel.ForEach(data, (value) => {
                ProcessData(value);
            });
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Parallel foreach: {0} milliseconds", elapsed);
        }

        //
        // Listing 13-3
        //
        static int Operation(int index) {
            DataCalculation(index);
            Console.WriteLine("Operation {0}", index);
            return index * 10;
        }
        static void DemoDo_Sequential() {
            Console.WriteLine("=== Sequential calls ===");
            Operation(1);
            Operation(2);
            Operation(3);
            Operation(4);
        }
        static void DemoDo_Parallel() {
            Console.WriteLine("=== Parallel.Do calls ===");
            Parallel.Do(
                () => Operation(1),
                () => Operation(2),
                () => Operation(3),
                () => Operation(4));
        }

        //
        // Listing 13-4
        //
        static void DemoTask() {
            Console.WriteLine("=== Task calls ===");
            Task t1 = Task.Create(delegate { Operation(1); });
            Task t2 = Task.Create(delegate { Operation(2); });
            Task t3 = Task.Create(delegate { Operation(3); });
            Task t4 = Task.Create(delegate { Operation(4); });
            TaskCoordinator.WaitAll( new Task[] { t1, t2, t3, t4 } );
        }

        //
        // Listing 13-5
        //
        static void DemoFuture() {
            Console.WriteLine("=== Future<T> calls ===");
            var f1 = Future.Create(() => Operation(1));
            var f2 = Future.Create(() => Operation(2));
            var f3 = Future.Create(() => Operation(3));
            var f4 = Future.Create(() => Operation(4));

            TaskCoordinator.WaitAll(new Task[] { f1, f2, f3, f4 });
            int result = f1.Value + f2.Value + f3.Value + f4.Value;
            Console.WriteLine("Future result is {0}", result);
        }

        //
        // Listing 13-6
        //
        static void DemoConcurrency_RaceCondition() {
            Console.WriteLine("=== DemoConcurrency_RaceCondition ==="); 
            int sum = 0;
            object sync = new object();
            Parallel.For(0, 10, (index) =>
            {
                int local = sum;
                local += Operation(index);
                sum = local;
            });
            Console.WriteLine("Final sum: {0}", sum);
        }

        //
        // Listing 13-7
        //
        static void DemoConcurrency_Safe() {
            Console.WriteLine("=== DemoConcurrency_Safe ===");
            int sum = 0;
            int[] results = new int[10];
            Parallel.For(0, 10, (index) =>
            {
                results[index] = Operation(index);
            });
            sum = results.Sum();
            Console.WriteLine("Final sum: {0}", sum);
        }


        //
        // Listing 13-8
        //
        static void StandardLinq() {
            Console.WriteLine("=== StandardLinq ===");
            int[] data = LoadData(100000000);
            var query =
                from   i in data
                where  i % 12345678 == 0
                select new { Value = i, ThreadID = Thread.CurrentThread.ManagedThreadId };

            Stopwatch sw = Stopwatch.StartNew();
            foreach (var number in query) {
                Console.WriteLine("{0} - from ThreadId={1}",
                                  number,
                                  Thread.CurrentThread.ManagedThreadId);
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

        //
        // Listing 13-9
        //
        static void SamplePLinq() {
            Console.WriteLine("=== SamplePLinq ===");
            int[] data = LoadData(100000000);
            var query =
                from   i in data.AsParallel()
                where  i % 12345678 == 0
                select new { Value = i, ThreadID = Thread.CurrentThread.ManagedThreadId };

            Stopwatch sw = Stopwatch.StartNew();
            foreach (var number in query) {
                Console.WriteLine("{0} - from ThreadId={1}",
                                  number,
                                  Thread.CurrentThread.ManagedThreadId);
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

        //
        // Listing 13-10
        //
        static void SamplePLinq_StopAndGo() {
            Console.WriteLine("=== SamplePLinq_StopAndGo ===");
            int[] data = LoadData(100000000);
            var query =
                from   i in data.AsParallel()
                where  i % 12345678 == 0
                select new { Value = i, ThreadID = Thread.CurrentThread.ManagedThreadId };

            Stopwatch sw = Stopwatch.StartNew();
            using (var e = query.GetEnumerator(false)) {
                // The first call to MoveNext waits for the execution 
                // of the entire query. After that call, data is enumerated 
                // from an in-memory list
                while (e.MoveNext()) {
                    Console.WriteLine("{0} - from ThreadId={1}",
                                      e.Current,
                                      Thread.CurrentThread.ManagedThreadId);
                }
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

        //
        // Listing 13-11
        //
        static void SamplePLinq_Inverted() {
            Console.WriteLine("=== SamplePLinq_Inverted ===");
            int[] data = LoadData(100000000);
            var query =
                from   i in data.AsParallel()
                where  i % 12345678 == 0
                select new { Value = i, ThreadID = Thread.CurrentThread.ManagedThreadId };

            Stopwatch sw = Stopwatch.StartNew();
            query.ForAll((number) => {
                Console.WriteLine("{0} - from ThreadId={1}",
                                  number,
                                  Thread.CurrentThread.ManagedThreadId);
            } );
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

        //
        // Listing 13-12
        //
        static void SamplePLinq_OrderBy() {
            Console.WriteLine("=== SamplePLinq_OrderBy ===");
            int[] data = LoadData(100000000);
            var query =
                from    i in data.AsParallel()
                where   i % 12345678 == 0
                orderby i
                select  new { Value = i, ThreadID = Thread.CurrentThread.ManagedThreadId };

            Stopwatch sw = Stopwatch.StartNew();
            foreach (var number in query) {
                Console.WriteLine("{0} - from ThreadId={1}",
                                  number,
                                  Thread.CurrentThread.ManagedThreadId);
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

     
        //
        // Listing 13-13 
        //
        static void SamplePLinq_PreserveOrdering() {
            Console.WriteLine("=== SamplePLinq_PreserveOrdering ===");
            int[] data = LoadData(100000000);
            var query =
                from   i in data.AsParallel()
                where  i % 12345678 == 0
                select new { Value = i, ThreadID = Thread.CurrentThread.ManagedThreadId };

            Stopwatch sw = Stopwatch.StartNew();
            foreach (var number in query) {
                Console.WriteLine("{0} - from ThreadId={1}",
                                  number,
                                  Thread.CurrentThread.ManagedThreadId);
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

        //
        // No listing in code (but sample is shown in a code block)
        //
        static void SamplePLinq_RaceCondition() {
            Console.WriteLine("=== SamplePLinq_RaceCondition ===");
            int[] data = LoadData(100);
            int accumulator = 0;
            var query =
                from i in data.AsParallel(ParallelQueryOptions.PreserveOrdering)
                select new { Aggregated = accumulator += i, 
                             Value = i,
                             ThreadID = Thread.CurrentThread.ManagedThreadId };
            Stopwatch sw = Stopwatch.StartNew();
            foreach (var number in query) {
                Console.WriteLine("{0} - from ThreadId={1}",
                                  number,
                                  Thread.CurrentThread.ManagedThreadId);
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

        //
        // No listing in code (but sample is shown in a code block)
        //
        static void SamplePLinq_LoadList_Safe() {
            Console.WriteLine("=== SamplePLinq_LoadList ===");
            int[] data = LoadData(1000);
            List<int> result = new List<int>();
            var query =
                from   i in data.AsParallel()
                where  i % 2 == 0
                select i;
            Stopwatch sw = Stopwatch.StartNew();
            foreach (var number in query) {
                result.Add(number);
            }
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

        static void SamplePLinq_LoadList_Unsafe() {
            Console.WriteLine("=== SamplePLinq_LoadList ===");
            int[] data = LoadData(1000);
            List<int> result = new List<int>();
            var query =
                from   i in data.AsParallel()
                where  i % 2 == 0
                select i;
            Stopwatch sw = Stopwatch.StartNew();
            query.ForAll((number) => {
                result.Add(number);
            });
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }

        static void SamplePLinq_ToList() {
            Console.WriteLine("=== SamplePLinq_LoadList ===");
            int[] data = LoadData(1000);
            var query =
                from   i in data.AsParallel()
                where  i % 2 == 0
                select i;
            Stopwatch sw = Stopwatch.StartNew();
            List<int> result = query.ToList();
            long elapsed = sw.ElapsedMilliseconds;
            Console.WriteLine("Elapsed time   : {0} milliseconds", elapsed);
            Console.WriteLine("CurrentThreadId: {0}", Thread.CurrentThread.ManagedThreadId);
        }


        //
        // Listing 13-14 
        //
        static void SamplePLinq_ExceptionHandling() {
            int[] data = LoadData(100);
            var query =
                from i in data.AsParallel()
                where i % 5 == 0
                select new { Value = i * 2 / (i % 10), ThreadID = Thread.CurrentThread.ManagedThreadId };

            try {
                foreach (var number in query) {
                    Console.WriteLine("{0} - from ThreadId={1}",
                                      number,
                                      Thread.CurrentThread.ManagedThreadId);
                }
            }
            catch (AggregateException ex) {
                Console.WriteLine(ex.Message);
                Console.WriteLine(
                    String.Format( "AggregateException with {0} inner exceptions", 
                                   ex.InnerExceptions.Count ) );
                int counter = 0;
                foreach (var innerException in ex.InnerExceptions) {
                    Console.WriteLine("({0})--> {1} : {2}", 
                                      ++counter, 
                                      innerException.GetType().Name,
                                      innerException.Message);
                }
            }
        }

        //
        // Listing 13-15
        //
        static void XmlPLinq() {
            XElement doc = XElement.Load(@"..\..\NorthwindCustomersWithOrders.xml");
            var query =
                from c in doc.Elements("customer").AsParallel()
                where c.Element("country").Value == "Italy"
                select new {
                    CompanyName = c.Element("companyName").Value,
                    ThreadID = Thread.CurrentThread.ManagedThreadId
                };

            foreach (var item in query) {
                Console.WriteLine("{0} - from ThreadId={1}",
                                  item,
                                  Thread.CurrentThread.ManagedThreadId);
            }
        }


        //
        // No listing in code (but sample is shown in a code block)
        //
        static void SqlPLinq() {
            DataContext db = new DataContext("Database=Northwind;Integrated Security=yes;Server=localhost");
            db.Log = Console.Out;
            var query =
                from row in
                    (from c in db.GetTable<Customer>()
                     where c.Country == "Italy"
                     select new { c.CompanyName }
                     ).AsParallel()
                select new {
                    row.CompanyName,
                    ThreadID = Thread.CurrentThread.ManagedThreadId
                };

            foreach (var item in query) {
                Console.WriteLine("{0} - from ThreadId={1}",
                                  item,
                                  Thread.CurrentThread.ManagedThreadId);
            }
        }

        static List<string> ScanTypeMethods(Type type, List<string> results) {
            if (results == null) {
                results = new List<string>();
            }
            foreach (var method in type.GetMethods()) {
                if (method.IsStatic
                    && method.ReturnType.GetInterface("IEnumerable") != null) {
                    string fullName = String.Format("{0}.{1}",
                                          method.DeclaringType.Name,
                                          method.Name);
                    if (results.IndexOf(fullName) < 0) {
                        results.Add(fullName);
                    }
                }
            }
            return results;
        }

        static List<string> ScanAssembly(Assembly assembly, List<string> results ) {
            if (results == null) {
                results = new List<string>();
            }
            foreach (var type in assembly.GetTypes()) {
                results = ScanTypeMethods(type, results);
            }
            return results;
        }
        
        static List<string> ScanAssemblies_Sequential() {
            List<string> results = new List<string>();
            foreach (var assembly in AppDomain.CurrentDomain.GetAssemblies()) {
                results = ScanAssembly(assembly, results);
            }
            return results;
        }

        static List<string> ScanAssemblies_Parallel() {
            List<string> results = new List<string>();
            Parallel.ForEach(
                AppDomain.CurrentDomain.GetAssemblies(),
                assembly => ScanAssembly(assembly, results));
            return results;
        }

        static void ReflectionScan_Sequential() {
            Console.WriteLine("Sequential Scan");
            for (int i = 0; i < 3; i++) {
                Stopwatch sw = Stopwatch.StartNew();
                var methods = ScanAssemblies_Sequential();
                Console.WriteLine("Test {0}: {1} milliseconds ({2} methods found)", i, sw.ElapsedMilliseconds, methods.Count);
            }
        }

        static void ReflectionScan_Parallel() {
            Console.WriteLine("Parallel Scan");
            for (int i = 0; i < 3; i++) {
                Stopwatch sw = Stopwatch.StartNew();
                var methods = ScanAssemblies_Parallel();
                Console.WriteLine("Test {0}: {1} milliseconds ({2} methods found)", i, sw.ElapsedMilliseconds, methods.Count);
            }
        }

        static void CompareReflectionScan() {
            ReflectionScan_Sequential();
            ReflectionScan_Parallel();
        }

        static void ReflectionSearch() {
            var query =
                from assembly in AppDomain.CurrentDomain.GetAssemblies() // .AsParallel()
                from type in assembly.GetTypes()
                from method in type.GetMethods()
                where method.IsStatic
                        && method.ReturnType.GetInterface("IEnumerable") != null
                orderby method.DeclaringType.Name, method.Name
                group method by new
                {
                    Class = method.DeclaringType.Name,
                    Method = method.Name
                };

            Stopwatch sw1 = Stopwatch.StartNew();
            var list1 = query.ToList();
            Console.WriteLine("{0} milliseconds", sw1.ElapsedMilliseconds);
            Stopwatch sw2 = Stopwatch.StartNew();
            var list2 = query.ToList();
            Console.WriteLine("{0} milliseconds", sw2.ElapsedMilliseconds);
            Stopwatch sw3 = Stopwatch.StartNew();
            var list3 = query.ToList();
            Console.WriteLine("{0} milliseconds", sw3.ElapsedMilliseconds);

            //foreach (var row in query) {
            //    Console.WriteLine("{0}.{1}", row.Key.Class, row.Key.Method);
            //}
        }
    }

    [Table(Name = "Customers")]
    public class Customer {
        [Column(IsPrimaryKey = true)]
        public string CustomerID;
        [Column]
        public string CompanyName;
        [Column]
        public string City;
        [Column(Name = "Region")]
        public string State;
        [Column]
        public string Country;
    }
}
