﻿Module ObjectInitializer

    Sub Main()
        ClassicalInitialization()
        NewSyntax()
        NewSyntaxMultiLine()
        NewSyntaxSingleLine()
        InitializerAndLocalTypeInference()
        Nested()
        NestedWith()
        Owned()
        OwnedWith()
        Collections()
    End Sub

    Sub ClassicalInitialization()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-12
        '
        Dim customer As New Customer
        With customer
            .Name = "Marco"
            .Country = "Italy"
        End With

        '
        ' END Listing C-12
        '
        ' ------------------------------------------------------
    End Sub

    Sub NewSyntax()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-13
        '
        Dim customer As Customer
        customer = New Customer With {.Name = "Marco", .Country = "Italy"}

        '
        ' END Listing C-13
        '
        ' ------------------------------------------------------
    End Sub
    Sub NewSyntaxMultiLine()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-14
        '
        Dim customer As Customer = _
            New Customer With { _
                .Name = "Marco", _
                .Country = "Italy"}
        '
        ' END Listing C-14
        '
        ' ------------------------------------------------------
    End Sub
    Sub NewSyntaxSingleLine()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-15
        '
        Dim customer = New Customer With {.Name = "Marco", .Country = "Italy"}
        '
        ' END Listing C-15
        '
        ' ------------------------------------------------------
    End Sub

    Sub InitializerAndLocalTypeInference()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-16
        '
        Dim c1 = New Customer With {.Name = "Marco", .Country = "Italy"}
        Dim c2 = New Customer("Paolo", 21) With {.Country = "Italy"}
        Dim c3 = New Customer With {.Name = "Paolo", .Age = 21, .Country = "Italy"}
        Dim c4 = New Customer With {.Name = c1.Name, .Country = c2.Country, .Age = c2.Age}
        '
        ' END Listing C-16
        '
        ' ------------------------------------------------------
    End Sub

    Sub Nested()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-17
        '
        Dim r = New Rectangle With { _
                    .TL = New Point With {.X = 0, .Y = 1}, _
                    .BR = New Point With {.X = 2, .Y = 3} _
                }
        '
        ' END Listing C-17
        '
        ' ------------------------------------------------------
    End Sub
    Sub NestedWith()
        Dim r = New Rectangle
        With r
            .TL = New Point
            With .TL
                .X = 0
                .Y = 1
            End With
            .BR = New Point
            With .BR
                .X = 2
                .Y = 3
            End With
        End With
        Console.WriteLine(r.ToString())
    End Sub
    Sub Owned()
        Dim r = New RectangleOwnedPoints With { _
                    .TL = New Point With {.X = 0, .Y = 1}, _
                    .BR = New Point With {.X = 2, .Y = 3} _
                }
        Console.WriteLine(r.ToString())
    End Sub
    Sub OwnedWith()
        Dim r = New RectangleOwnedPoints
        With r
            With .TL
                .X = 0
                .Y = 1
            End With
            With .BR
                .X = 2
                .Y = 3
            End With
        End With
        Console.WriteLine(r.ToString())
    End Sub
    Sub Arrays()
        Dim integers As Integer() = {1, 3, 9, 18}
        Dim customers As String() = {"Jack", "Paolo", "Marco"}
    End Sub
    Sub Collections()
        'Collection initializer are no more supported in VB 9.0
        'Commented BEGIN
        'Dim integers = New List(Of Integer)with { 1, {3}, {9}, {18}}
        'Dim integers = New List(Of Integer) { 1, 3, 9, 18}
        'Dim customers = New List(Of Customer) { _
        '        {Name := "Jack", Age := 28, Country := "USA"}, _
        '        New Customer {Name := "Paolo"}, _
        '        {Name := "Marco", Country := "Italy"}}
        'For Each c In customers
        '    Console.WriteLine(c.Name)
        'Next
        'Commented END
    End Sub
End Module


' ------------------------------------------------------
'
' BEGIN Listing C-11
'
Public Class Customer
    Public Age As Integer
    Public Name As String
    Public Country As String
    Public Sub New(ByVal name As String, ByVal age As Integer)
        Me.Age = age
        Me.Name = name
    End Sub
    Public Sub New()
        Age = -1
        Name = "<Unknown>"
        Country = "<Unknown>"
    End Sub

End Class
'
' END Listing C-11
'
' ------------------------------------------------------


' ------------------------------------------------------
'
' BEGIN Listing C-17
'
Public Class Point
    Private _x, _y As Integer

    Public Property X() As Integer
        Get
            Return _x
        End Get
        Set(ByVal value As Integer)
            _x = value
        End Set
    End Property
    Public Property Y() As Integer
        Get
            Return _y
        End Get
        Set(ByVal value As Integer)
            _y = value
        End Set
    End Property
    Public Overrides Function ToString() As String
        Return X.ToString() & "," & Y.ToString()
    End Function
End Class

Public Class Rectangle
    Private _tl, _br As Point
    Public Property TL() As Point
        Get
            Return _tl
        End Get
        Set(ByVal value As Point)
            _tl = value
        End Set
    End Property
    Public Property BR() As Point
        Get
            Return _br
        End Get
        Set(ByVal value As Point)
            _br = value
        End Set
    End Property
    Public Overrides Function ToString() As String
        Return "(" & TL.ToString() & ")-(" & BR.ToString() & ")"
    End Function
End Class
'
' END Listing C-17
'
' ------------------------------------------------------



Public Class RectangleOwnedPoints
    Private _tl As Point = New Point()
    Private _br As Point = New Point()
    Public Property TL() As Point
        Get
            Return _tl
        End Get
        Set(ByVal value As Point)
            _tl = value
        End Set
    End Property
    Public Property BR() As Point
        Get
            Return _br
        End Get
        Set(ByVal value As Point)
            _br = value
        End Set
    End Property
    Public Overrides Function ToString() As String
        Return "(" & TL.ToString() & ")-(" & BR.ToString() & ")"
    End Function
End Class
