﻿Class Employee
    Public Name As String
    Public ID As Long
    Public Department As String
End Class

Module XmlSupport

    Sub Main()
        'XmlExpression()             ' Listing C-30
        'XmlStandard()
        'XmlErrors()                 ' Listing C-31
        'XmlDynamic()                ' Listing C-32
        'XmlFullDynamic()            ' Listing C-33
        'XmlFullDynamicExtended()    ' Listing C-34
        'XmlClosingTag()             ' Listing C-35
        'XmlNestedConstants()
        'XmlLinecontinuation()
        'XmlNestedConstantsList()    ' Listing C-36
        'XmlNestedQuery()            ' Listing C-37
        'XmlAccess()                 ' Listing C-38, Listing C-39, Listing C-40

    End Sub

    Sub XmlExpression()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-31
        '
        Dim book As XElement
        book = _
            <Book Title="Introducing LINQ">
                <Author>Marco Russo</Author>
                <Author>Paolo Pialorsi</Author>
            </Book>
        '
        ' END Listing C-31
        '
        ' ------------------------------------------------------

        'Console.WriteLine(book)
    End Sub

    Public Sub XmlExpression_commonstyle()
        Dim book As XElement
        book = New XElement("Book", _
                    New XAttribute("Title", "Introducing LINQ"), _
                    New XElement("Author", "Marco Russo"), _
                    New XElement("Author", "Paolo Pialorsi"))


        'Console.WriteLine(book)
    End Sub
    Public Sub XmlExpression_optimized()
        Dim book As New XElement(XName.Get("Book", ""))
        book.Add(New XAttribute(XName.Get("Title", ""), "Introducing LINQ"))
        Dim author As New XElement(XName.Get("Author", ""))
        author.Add("Marco Russo")
        book.Add(author)
        author = New XElement(XName.Get("Author", ""))
        author.Add("Paolo Pialorsi")
        book.Add(author)

        'Console.WriteLine(book)
    End Sub



    Sub XmlStandard()
        Dim book As XElement
        book = New XElement("Book", _
                New XAttribute("Title", "Introducing LINQ"), _
                New XElement("Author", "Marco Russo"), _
                New XElement("Author", "Paolo Pialorsi"))
        Console.WriteLine(book)
    End Sub

    Sub XmlErrors()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-32
        '

        'Dim a = <Book Title="Invalid Title></Book>
        'Dim b = <Book Title="Good Title"><Book>

        '
        ' END Listing C-32
        '
        ' ------------------------------------------------------
    End Sub

    Sub XmlDynamic()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-33
        '

        Dim bookTitle = "Introducing LINQ"
        Dim author1 = "Marco Russo"
        Dim author2 = "Paolo Pialorsi"
        Dim book = _
            <Book Title=<%= bookTitle %>>
                <Author><%= author1 %></Author>
                <Author><%= author2 %></Author>
            </Book>
        '
        ' END Listing C-33
        '
        ' ------------------------------------------------------
        Console.WriteLine(book)
    End Sub

    Sub XmlFullDynamic()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-34
        '
        Dim bookTitle = "Introducing LINQ"
        Dim author1 = "Marco Russo"
        Dim author2 = "Paolo Pialorsi"
        Dim tagBook = "Book"
        Dim attrName = "Title"
        Dim tagAuthors = "Authors"
        Dim book = _
            <<%= tagBook %> <%= attrName %>=<%= bookTitle %>>
                <<%= tagAuthors %>><%= author1 & ", " & author2 %></>
            </>
        '
        ' END Listing C-34
        '
        ' ------------------------------------------------------
        Console.WriteLine(book)
    End Sub

    Sub XmlFullDynamicExtended()
        Dim bookTitle = "Introducing LINQ"
        Dim author1 = "Marco Russo"
        Dim author2 = "Paolo Pialorsi"
        Dim tagBook = "Book"
        Dim attrName = "Title"
        Dim tagAuthors = "Authors"
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-35
        '
        Dim book = _
              New XElement(tagBook, _
                  New XAttribute(attrName, bookTitle), _
                  New XElement(tagAuthors, author1 & ", " & author2))
        '
        ' END Listing C-35
        '
        ' ------------------------------------------------------
        Console.WriteLine(book)
    End Sub

    Sub XmlClosingTag()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-36
        '
        Dim tagElement = "Description"
        Dim sample = <<%= tagElement %>>Sample element</>
        '
        ' END Listing C-36
        '
        ' ------------------------------------------------------
    End Sub

    Sub XmlNestedConstants()
        Dim book = _
            <Book Title="Introducing LINQ">
                <%= <Publisher>Microsoft Press</Publisher> %>
            </Book>
        Console.WriteLine(book)
    End Sub
    Sub XmlLinecontinuation()
        Dim book = _
        <Book Title="Introducing LINQ">
            <%= _
                <Publisher>Microsoft Press</Publisher> _
            %>
        </Book>
        Console.WriteLine(book)
    End Sub
    Sub XmlNestedConstantsList()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-37

        Dim book = _
            <Book Title="Introducing LINQ">
                <%= New XElement() { _
                    <Author>Marco Russo</Author>, _
                    <Author>Paolo Pialorsi</Author>} %>
            </Book>

        ' END Listing C-37
        '
        ' ------------------------------------------------------
        Console.WriteLine(book)
    End Sub

    Sub XmlNestedQuery()
        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-38
        '
        Dim team() = {New With {.Name = "Marco Russo", .Role = "Author"}, _
                      New With {.Name = "Paolo Pialorsi", .Role = "Author"}, _
                      New With {.Name = "Roberto Brunetti", .Role = "Reviewer"}}
        Dim book = _
            <Book Title="Introducing LINQ">
                <%= From person In team _
                    Where person.Role = "Author" _
                    Select <Author><%= person.Name %></Author> %>
            </Book>
        '
        ' END Listing C-38
        '
        ' ------------------------------------------------------
        Console.WriteLine(book)
    End Sub

    Sub XmlAccess()
        Dim movies = _
        <Movies>
            <Movie Title="Pulp Fiction">
                <Genre>Crime</Genre>
                <Director>Quentin Tarantino</Director>
            </Movie>
            <Movie Title="Reservoir Dogs">
                <Genre>Crime</Genre>
                <Genre>Drama</Genre>
                <Genre>Thriller</Genre>
                <Director>Quentin Tarantino</Director>
            </Movie>
            <Movie Title="Fight Club">
                <Genre>Crime</Genre>
                <Genre>Drama</Genre>
                <Genre>Thriller</Genre>
                <Director>David Fincher</Director>
            </Movie>
            <Movie Title="Alien 3">
                <Genre>Science Fiction</Genre>
                <Genre>Action</Genre>
                <Genre>Horror</Genre>
                <Genre>Drama</Genre>
                <Genre>Thriller</Genre>
                <Director>David Fincher</Director>
            </Movie>
        </Movies>

        'Console.WriteLine(movies)

        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-39
        ' 
        Dim fightClub = _
            (From movie In movies.<Movie> _
            Where movie.@Title = "Fight Club" _
            Select movie).First()

        ' Get the first genre
        Dim firstGenre = fightClub.<Genre>(0).Value
        ' Corresponds to: firstGenre = fightClub.Elements("Genre")(0).Value
        Console.WriteLine("First = {0}", firstGenre)

        ' Display all genres
        ' Corresponds to: fightClub.Elements("Genre")
        For Each g In fightClub.<Genre>
            Console.WriteLine(g.Value)
        Next
        '
        ' END Listing C-39
        '
        ' ------------------------------------------------------

        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-40
        ' 
        ' Display the Title attribute of Fight Club movie
        Console.WriteLine(fightClub.@Title)
        ' Corresponds to: Console.WriteLine(fightClub.Attribute("Title").Value)
        '
        ' END Listing C-40
        '
        ' ------------------------------------------------------

        ' ------------------------------------------------------
        '
        ' BEGIN Listing C-41
        ' 
        ' List of Directors
        Dim directors = movies...<Director>
        For Each director In directors
            Console.WriteLine(director)
        Next
        '
        ' END Listing C-41
        '
        ' ------------------------------------------------------


    End Sub

End Module
