﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevLeap.Linq.WCF.LinqToEntities.Contracts;
using NorthwindModel;
using System.Data.Objects;
using System.Data.Metadata.Edm;

namespace DevLeap.Linq.WCF.LinqToEntities.Biz
{
    public class CustomersBiz
    {
        private NorthwindEntities getContext()
        {
            return(new NorthwindModel.NorthwindEntities());
        }

        public CustomersList ListCustomers(CustomersFilter request)
        { 
            // Enforce security checks
            // TBD...

            // Enforce validation rules
            // TBD...

            var northwind = getContext();

            // Execute the request
            var customers =
                from c in northwind.Customers
                where (
                    (String.IsNullOrEmpty(request.Country) || c.Country == request.Country) &&
                    (String.IsNullOrEmpty(request.Region) || c.Region == request.Region))
                select c;

            return (new CustomersList(customers));
        }

        public Customers GetCustomer(String customerID)
        {
            var northwind = getContext();

            var customer =
                (from c in northwind.Customers
                where c.CustomerID == customerID
                select c).First();

            return (customer);
        }

        public Customers UpdateCustomer(Customers customer)
        {
            var northwind = getContext();

            northwind.AttachTo("Customers", customer);

            // These lines to force the entity as modified
            // because it is marked as "unchanged" just after AttachTo
            ObjectStateEntry entry = northwind.ObjectStateManager.GetObjectStateEntry(customer.EntityKey);
            if (entry != null)
                entry.SetModified();

            // Refresh the entity keeping consumer-side changes
            northwind.Refresh(RefreshMode.ClientWins, customer);

            //// Alternative solution: 
            //// mark each property on the entity as modified
            //foreach (var p in northwind.MetadataWorkspace.GetItem<EntityType>(customer.GetType().FullName, System.Data.Metadata.Edm.DataSpace.CSpace).Properties)
            //{
            //    entry.SetModifiedProperty(p.Name);
            //}

            northwind.SaveChanges(true);
            
            return(customer);
        }

        public Customers InsertCustomer(Customers customer)
        {
            var northwind = getContext();

            northwind.AddObject("Customers", customer);
            northwind.SaveChanges(true);
            
            return (customer);
        }

        public Boolean DeleteCustomer(Customers customer)
        {
            var northwind = getContext();

            northwind.AttachTo("Customers", customer);
            northwind.DeleteObject(customer);
            northwind.SaveChanges(true);

            return (true);
        }
    }
}
