﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using DevLeap.Linq.WCF.LinqToEntities.Contracts;
using NorthwindModel;

namespace DevLeap.Linq.WCF.LinqToEntities.ConsumerByFactory
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Press ENTER to start CustomersService consumer");
            Console.ReadLine();

            CustomersServiceConsumer();

            Console.WriteLine("\nPress ENTER to start CustomerManagerService consumer");
            Console.ReadLine();

            CustomerManagerConsumer();
        }

        private static void CustomersServiceConsumer()
        {
            ChannelFactory<ICustomersService> factory =
                new ChannelFactory<ICustomersService>("CustomersServiceEndpoint");

            ICustomersService channel = factory.CreateChannel();

            try
            {
                CustomersFilter request = new CustomersFilter { Country = "USA" };
                var result = channel.ListCustomers(request);

                foreach (Customers c in result)
                {
                    Console.WriteLine("{0} - {1} - {2}", c.CustomerID,
                        c.CompanyName, c.Country);
                }
            }
            finally
            {
                if (((ICommunicationObject)channel).State != CommunicationState.Closed) 
                    ((ICommunicationObject)channel).Close();
            }
        }

        private static void CustomerManagerConsumer()
        {
            String customerID = "ALFKI";

            ChannelFactory<ICustomerManagerService> factory =
                new ChannelFactory<ICustomerManagerService>("CustomerManagerEndpoint");

            ICustomerManagerService channel = factory.CreateChannel();

            try
            {
                var customer = channel.GetCustomer(new CustomerKey { CustomerID = customerID });

                Console.WriteLine("Customer {0} has country: {1}",
                    customer.CustomerID, customer.Country);

                Console.WriteLine("Press ENTER to change customer country");
                Console.ReadLine();

                customer.Country = "Italy";
                customer = channel.UpdateCustomer(customer);

                Console.WriteLine("Customer {0} now has country: {1}",
                    customer.CustomerID, customer.Country);

                Console.WriteLine("Press ENTER to change back customer country");
                Console.ReadLine();

                customer.Country = "Germany";
                customer = channel.UpdateCustomer(customer);

                Console.WriteLine("Customer {0} now has country: {1}",
                    customer.CustomerID, customer.Country);

                Console.WriteLine("Press ENTER to insert a new customer");
                Console.ReadLine();

                Customers newCustomer = new Customers
                {
                    CustomerID = "PAOLO",
                    ContactName = "Paolo Pialorsi",
                    CompanyName = "DevLeap"
                };

                newCustomer = channel.InsertCustomer(newCustomer);

                Console.WriteLine("New customer inserted");

                Console.WriteLine("Press ENTER to delete the new customer");
                Console.ReadLine();

                DeletionResult result = channel.DeleteCustomer(newCustomer);
                Console.WriteLine("Deletion result: {0}", result.Result);
            }
            finally
            {
                if (((ICommunicationObject)channel).State != CommunicationState.Closed)
                    ((ICommunicationObject)channel).Close();
            }
        }
    }
}
