﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml;
using System.IO;
using System.Runtime.Serialization;
using System.ComponentModel;
using System.Runtime.Serialization.Formatters.Soap;

namespace DevLeap.Linq.WCF.LinqToEntitiesDataContract
{
    class Program
    {
        static void Main(string[] args)
        {
            NorthwindModel.NorthwindEntities nw = new NorthwindModel.NorthwindEntities();

            var customersFromUSA =
                from c in nw.Customers
                where c.Country == "USA" && c.Orders.Count > 3
                select c;

            var orderDetailWithProduct =
                (from o in nw.Orders
                from d in o.Order_Details
                join p in nw.Products on d.ProductID equals p.ProductID
                select new {d.OrderID, d.ProductID, p.CategoryID, p.ProductName }).First();

            foreach (var c in customersFromUSA)
            {
                ((INotifyPropertyChanged)c).PropertyChanged += new PropertyChangedEventHandler(Program_PropertyChanged);

                Console.WriteLine("{0} => {1}", c.CompanyName, c.ContactName);
                c.Orders.Load();
                var order = c.Orders.First();

                ((INotifyPropertyChanged)order).PropertyChanged += new PropertyChangedEventHandler(Program_PropertyChanged);

                order.OrderDate = DateTime.Now;

                Console.WriteLine(c.EntityState);
                Console.WriteLine(order.EntityState);
            }

            try
            {
                DataContractSerializer dcs = new DataContractSerializer(customersFromUSA.ToList().GetType());
                dcs.WriteObject(new XmlTextWriter(Console.Out), customersFromUSA.ToList());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            try
            {
                IFormatter formatter = new BinaryFormatter();
                MemoryStream mem = new MemoryStream();
                formatter.Serialize(mem, customersFromUSA.ToList().First());

                mem.Position = 0;
                StreamReader sr = new StreamReader(mem);
                Console.WriteLine(sr.ReadToEnd());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }

        static void Program_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            Console.WriteLine(e.PropertyName, sender.ToString());
        }
    }
}
