﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.IO;
using System.Xml;
using System.Runtime.Serialization.Formatters.Binary;
using DevLeap.Linq.WCF.LinqToSqlDataContract.Serialization;

namespace DevLeap.Linq.WCF.LinqToSqlDataContract
{
    class Program
    {
        static void Main(string[] args)
        {
            //TrySerializationOfNativeLinqToSqlEntities();
            //TrySerializationOfListOfNativeLinqToSqlEntities();
            //TrySerializationOfAnonymousTypesListFromLinqToSql();
            //TrySerializationOfUserDefinedEntities();
            //TrySerializationOfObjectGraph();
        }

        static void TrySerializationOfNativeLinqToSqlEntities()
        {
            NorthwindModelDataContext dc = new NorthwindModelDataContext(
                "server=localhost;database=Northwind;integrated security=SSPI;");

            dc.Log = Console.Out;

            var customersFromUSA =
                from c in dc.Customers
                where c.Country == "USA" && c.Orders.Count > 3
                select c;

            try
            {
                BinaryFormatter formatter = new BinaryFormatter();
                MemoryStream mem = new MemoryStream();
                formatter.Serialize(mem, customersFromUSA);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            Console.ReadLine();

            try
            {
                DataContractSerializer dcs = new DataContractSerializer(customersFromUSA.GetType());
                dcs.WriteObject(new XmlTextWriter(Console.Out), customersFromUSA);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }

        static void TrySerializationOfListOfNativeLinqToSqlEntities()
        {
            NorthwindModelDataContext dc = new NorthwindModelDataContext(
                "server=localhost;database=Northwind;integrated security=SSPI;");

            dc.Log = Console.Out;

            var customersFromUSA =
                from c in dc.Customers
                where c.Country == "USA" && c.Orders.Count > 3
                select c;

            try
            {
                BinaryFormatter formatter = new BinaryFormatter();
                MemoryStream mem = new MemoryStream();
                formatter.Serialize(mem, customersFromUSA.ToList());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            Console.ReadLine();

            try
            {
                DataContractSerializer dcs = new DataContractSerializer(customersFromUSA.ToList().GetType());
                dcs.WriteObject(new XmlTextWriter(Console.Out), customersFromUSA.ToList());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }

        static void TrySerializationOfAnonymousTypesListFromLinqToSql()
        {
            NorthwindModelDataContext dc = new NorthwindModelDataContext(
                "server=localhost;database=Northwind;integrated security=SSPI;");

            dc.Log = Console.Out;

            var customersFromUSA =
                from c in dc.Customers
                where c.Country == "USA" && c.Orders.Count > 3
                select new { c.CustomerID, c.ContactName, c.CompanyName };

            try
            {
                BinaryFormatter formatter = new BinaryFormatter();
                MemoryStream mem = new MemoryStream();
                formatter.Serialize(mem, customersFromUSA.ToList());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            Console.ReadLine();

            try
            {
                DataContractSerializer dcs = new DataContractSerializer(customersFromUSA.ToList().GetType());
                dcs.WriteObject(new XmlTextWriter(Console.Out), customersFromUSA.ToList());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }

        static void TrySerializationOfUserDefinedEntities()
        {
            NorthwindModelDataContext dc = new NorthwindModelDataContext(
                "server=localhost;database=Northwind;integrated security=SSPI;");

            dc.Log = Console.Out;

            var customersFromUSA =
                from c in dc.Customers
                where c.Country == "USA" && c.Orders.Count > 3
                select new SerializableCustomer()
                {
                    CustomerID = c.CustomerID,
                    CompanyName = c.CompanyName,
                    ContactName = c.ContactName
                };

            foreach (var c in customersFromUSA)
            {
                Console.WriteLine("{0} => {1}", c.CompanyName, c.ContactName);
            }

            try
            {
                BinaryFormatter formatter = new BinaryFormatter();
                MemoryStream mem = new MemoryStream();
                formatter.Serialize(mem, customersFromUSA.ToList());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            Console.ReadLine();

            try
            {
                DataContractSerializer dcs = new DataContractSerializer(customersFromUSA.ToList().GetType());
                dcs.WriteObject(new XmlTextWriter(Console.Out), customersFromUSA.ToList());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }

        static void TrySerializationOfObjectGraph()
        {
            NorthwindModelDataContext dc = new NorthwindModelDataContext(
                "server=localhost;database=Northwind;integrated security=SSPI;");

            dc.DeferredLoadingEnabled = true;
            dc.Log = Console.Out;

            var firstDetail =
                (from c in dc.Customers
                from o in c.Orders
                from d in o.Order_Details
                select d).First();

            Console.WriteLine(firstDetail.OrderID);

            try
            {
                BinaryFormatter formatter = new BinaryFormatter();
                MemoryStream mem = new MemoryStream();
                formatter.Serialize(mem, firstDetail);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            Console.ReadLine();

            try
            {
                DataContractSerializer dcs = new DataContractSerializer(firstDetail.GetType());
                dcs.WriteObject(new XmlTextWriter(Console.Out), firstDetail);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}
